/*
 * Licensed Materials - Property of IBM
 * pidnumber
 * (C) Copyright IBM Corporation 2005. All Rights Reserved.
 * US Government Users Restricted Rights- Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 */
package com.ibm.j2ca.sample.twineball;

import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.resource.ResourceException;

import com.ibm.despi.Accessor;
import com.ibm.despi.Cursor;
import com.ibm.despi.InputAccessor;
import com.ibm.despi.OutputAccessor;
import com.ibm.despi.exception.DESPIException;
import com.ibm.despi.exception.GetFailedException;
import com.ibm.despi.exception.SetFailedException;
import com.ibm.despi.exception.DESPIException;
import com.ibm.j2ca.extension.metadata.Property;
import com.ibm.j2ca.extension.metadata.Type;
import com.ibm.j2ca.extension.metadata.exceptions.InvalidMetadataException;

/**
 * ObjectNaming
 *
 * This class is intended to read metadata from business objects, and to be able
 * to set and unset keys inside business objects.
 */
public class ObjectNaming {
	private static final String NAMESPACE = TwineBallConstants.METADATA_NAMESPACE;

	private static final String FIELDNAME = "FieldName";

	private static final String OBJECTNAME = "ObjectName";

	public ObjectNaming(TwineBallResourceAdapter resourceAdapter) {
//		logger = resourceAdapter.getLogUtils();
//		this.resourceAdapter = resourceAdapter;
	}
	public ObjectNaming() {
//		logger = resourceAdapter.getLogUtils();
//		this.resourceAdapter = resourceAdapter;
	}

	/**
	 * getEntityName - get the Native/TwineBall name of a given DataObject
	 *
	 * @param dataObject -
	 *            the dataObject we want to get the name of
	 * @return the object's name
	 * @throws ResourceException
	 */
	public String getEntityName(Type metadata) throws ResourceException {
		String name = null;
		try {
			//trace(Level.FINEST, ObjectNaming.class.getName(), "getEntityName()", "Attempting to get entity name of" + metadata.getName());
			Map annotations = metadata.getAnnotations(NAMESPACE);
			name = (String) annotations.get(OBJECTNAME);
			//trace(Level.FINEST, ObjectNaming.class.getName(), "getEntityName()", "Name is " + name);
		} catch (InvalidMetadataException e) {
			throw new ResourceException(e);
		}
		return name;
	}

	/**
	 * getRealColumnNames
	 *
	 * @param dataObject
	 *            the dataObject we want the column names of
	 * @return a hashmap that maps the dataObject attribute's name to the
	 *         native/twineball name of that column
	 * @throws ResourceException
	 */
	public HashMap getRealColumnNames(Type metadata) throws ResourceException {
		HashMap attributeASI = new HashMap();
		try {
			Iterator attributeIterator = metadata.getPropertyIterator();
			while (attributeIterator.hasNext()) {
				Property thisProperty = (Property) attributeIterator.next();
				String propertyName = thisProperty.getName();
				//logger.trace(Level.FINEST, ObjectNaming.class.getName(), "getRealColumnNames()", "Attempting to get column name of attribute: " + propertyName);
				Map attributeDO = thisProperty.getAnnotations(NAMESPACE);
				if (attributeDO != null) {
					String columnName = (String) attributeDO.get(FIELDNAME);
					attributeASI.put(propertyName, columnName.toUpperCase());
				}
			}
		} catch (InvalidMetadataException e) {
			throw new ResourceException(e);
		}
		return attributeASI;
	}

	/**
	 * getKey - get the key of this data object as a string.
	 *
	 * @param dataObject
	 *            -the dataObject for which we want the key
	 * @return the key
	 * @throws ResourceException
	 */
	public String getKey(Type metadata, Cursor data) throws ResourceException {
		String keyValue;
		try {
			List keyProperties = metadata.getKeyProperties(NAMESPACE);
			String propName = ((Property) keyProperties.get(0)).getName();
			InputAccessor accessor = (InputAccessor)data.getAccessor(propName);
			keyValue = accessor.getString();
		} catch (InvalidMetadataException e) {
			throw new ResourceException(e);
		} catch (GetFailedException e) {
			throw new ResourceException(e);
		} catch (DESPIException e) {
			throw new ResourceException(e);
		}

		return keyValue;
	}

	/**
	 * setKey - set the key of a given data object
	 *
	 * @param dataObject
	 * @param key
	 * @throws ResourceException
	 */
	public void setKey(Type metadata, Cursor data, String key) throws ResourceException {
		try {
			List keyProperties = metadata.getKeyProperties(NAMESPACE);
			String propName = ((Property) keyProperties.get(0)).getName();
			Accessor accessor = data.getAccessor(propName);
			((OutputAccessor)accessor).setString(key);
		} catch (InvalidMetadataException e) {
			throw new ResourceException(e);
		} catch (SetFailedException e) {
			throw new ResourceException(e);
		} catch (DESPIException e) {
			throw new ResourceException(e);
		}
	}

	/**
	 * @return Returns the logger.
	 */
//	public LogUtils getLogger() {
//		return logger;
//	}
//
//	/**
//	 * @param logger
//	 *            The logger to set.
//	 */
//	public void setLogger(LogUtils logger) {
//		this.logger = logger;
//	}
//
//	/**
//	 * @return Returns the resourceAdapter.
//	 */
//	public TwineBallResourceAdapter getResourceAdapter() {
//		return resourceAdapter;
//	}
//
//	/**
//	 * @param resourceAdapter
//	 *            The resourceAdapter to set.
//	 */
//	public void setResourceAdapter(TwineBallResourceAdapter resourceAdapter) {
//		this.resourceAdapter = resourceAdapter;
//	}
}