/*
 * Licensed Materials - Property of IBM
 * pidnumber
 * (C) Copyright IBM Corporation 2005. All Rights Reserved.
 * US Government Users Restricted Rights- Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 */
package com.ibm.j2ca.sample.twineball;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Vector;

import javax.resource.ResourceException;

import com.ibm.despi.Cursor;
import com.ibm.despi.InputAccessor;
import com.ibm.despi.OutputAccessor;
import com.ibm.despi.exception.*;
import com.ibm.j2ca.extension.metadata.Property;
import com.ibm.j2ca.extension.metadata.Type;
import com.ibm.j2ca.extension.metadata.exceptions.InvalidMetadataException;

/**
 * ObjectSerializer. This class can convert an TwineBall object to a DataObject
 * and vice-versa. A TwineBall object is a Vector of name-value pairs.
 */
public class ObjectSerializer {
    ObjectNaming objectNaming;
    //LogUtils logger;
   // TwineBallResourceAdapter ra;

    /**
     * Constructor for ObjectSerializer
     *
     * @param objectNaming
     */
    public ObjectSerializer(ObjectNaming objectNaming) {
        this.objectNaming = objectNaming;
      //  this.logger = objectNaming.getLogger();
      //  this.ra = objectNaming.getResourceAdapter();
    }

    /**
     * toNameValueVector - convert the dataObject to a TwineBall object
     *
     * @param cursor
     * @return the twineBall object
     * @throws ResourceException
     */
    public Vector toNameValueVector(Type metadata, Cursor cursor) throws ResourceException {
        Vector returnVector = new Vector();
    	try{
      //  logger.trace(Level.FINER, ObjectSerializer.class.getName(), "toNameValueVector", "Serializing cursor: " + metadata.getName());

        HashMap realColumnNames = objectNaming.getRealColumnNames(metadata);
        Iterator propertyIterator = metadata.getPropertyIterator();
        while (propertyIterator.hasNext()) {
            Property currentProperty = (Property) propertyIterator.next();
            if (currentProperty.isContainment() || currentProperty.isMany())
                continue;
            String propertyName = currentProperty.getName();
            InputAccessor accessor = (InputAccessor)cursor.getAccessor(propertyName);
            if (accessor.isSet()){
            	String currentValue = accessor.getString();
            	String EISPropertyName = (String) realColumnNames.get(propertyName);
            	if ((currentValue != null) && (!currentValue.equals(""))) {
            		returnVector.add(EISPropertyName + "=" + currentValue.toString());
            	}
            }
        }
    	}catch(InvalidMetadataException e){
    		throw new ResourceException(e);
    	}catch(GetFailedException e){
    		throw new ResourceException(e);
    	}catch(DESPIException e){
    		throw new ResourceException(e);
    	}
        return returnVector;
    }


    /**
     * toSDO - convert the twineball object to a data object.
     *
     * @param nameValueVector -
     *            the input twineball object
     * @param cursor -
     *            an empty data object to populate.
     * @return the data object populated with the correct values
     * @throws ResourceException
     */
    public Cursor toCursor(Vector nameValueVector, Type metadata, Cursor  cursor) throws ResourceException {
        HashMap nameMapping = new HashMap();
        try{
        HashMap propertiesToRealColumns = objectNaming.getRealColumnNames(metadata);
        Iterator propertyIterator = metadata.getPropertyIterator();
        while (propertyIterator.hasNext()) {
            Property property = (Property) propertyIterator.next();
            String propertyName = property.getName();
            String EISPropertyName = (String) propertiesToRealColumns.get(propertyName);
            nameMapping.put(EISPropertyName, propertyName);
        }
        String currentElement = "";
        for (int i = 0; i < nameValueVector.size(); i++) {
            currentElement = (String) nameValueVector.elementAt(i);
            String column = currentElement.substring(0, currentElement.indexOf('='));
            String value = currentElement.substring(currentElement.indexOf('=') + 1);
            String columnName = (String) nameMapping.get(column.toUpperCase());
            if (columnName == null) {
     //           logger.trace(Level.FINER, ObjectSerializer.class.getName(), "toSDO", "no binding for column " + columnName);
                continue;
            }
            if ((value != null) && (!(value.equalsIgnoreCase("null")))) {
            	OutputAccessor accessor = (OutputAccessor)cursor.getAccessor(columnName);
                accessor.setString(value);
            }
        }
        }catch(InvalidMetadataException e){
        	throw new ResourceException(e);
        }catch(SetFailedException e){
        	throw new ResourceException(e);
        }catch(DESPIException e){
        	throw new ResourceException(e);
        }
        return cursor;
    }
}