/*
 * Licensed Materials - Property of IBM
 * pidnumber
 * (C) Copyright IBM Corporation 2005. All Rights Reserved.
 * US Government Users Restricted Rights- Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 */
package com.ibm.j2ca.sample.twineball;

import java.util.logging.Level;

import javax.resource.ResourceException;
import javax.resource.spi.BootstrapContext;
import javax.resource.spi.ResourceAdapterInternalException;

import com.ibm.j2ca.base.WBIActivationSpecWithXid;
import com.ibm.j2ca.base.WBIPollableResourceAdapterWithXid;
import com.ibm.j2ca.base.WBIResourceAdapter;
import com.ibm.j2ca.base.WBIResourceAdapterMetadata;
import com.ibm.j2ca.extension.eventmanagement.EventStoreWithXid;
import com.ibm.j2ca.extension.logging.LogUtilConstants;
import com.ibm.j2ca.sample.twineball.inbound.TwineBallActivationSpecWithXid;
import com.ibm.j2ca.sample.twineball.inbound.TwineBallEventStoreWithXid;


/**
 * TwineBallResourceAdapter. This is the lifecycle, and global property
 * container for the twineball adapter.
 */
public class TwineBallResourceAdapter extends WBIResourceAdapter implements WBIPollableResourceAdapterWithXid {
	
	public final String CLASSNAME="com.ibm.j2ca.sample.twineball.TwineBallResourceAdapter";
	
	/**
     * start.
     * 
     * @param bootstrapCtx
     * @throws ResourceAdapterInternalException
     * @see javax.resource.spi.ResourceAdapter#start(javax.resource.spi.BootstrapContext)
     */
    public void start(BootstrapContext bootstrapCtx) throws ResourceAdapterInternalException {
    	
        super.start(bootstrapCtx);
        this.getLogUtils().log(Level.INFO, LogUtilConstants.ADAPTER_RBUNDLE, TwineBallResourceAdapter.class.getName(), "start()", "1001", new Object[] {});
    }

    /**
     * createEventStore
     * 
     * @param activationSpec -
     *            inbound properties
     * @return an instance of TwineBallEventStore, ready for polling
     * @throws ResourceException
     * @see com.ibm.j2ca.base.WBIPollableResourceAdapter#createEventStore(javax.resource.spi.ActivationSpec)
     */
    public EventStoreWithXid createEventStore(WBIActivationSpecWithXid activationSpec) throws ResourceException {
        getLogUtils().traceMethodEntrance(TwineBallEventStoreWithXid.class.getName(), "createEventStore()");
        EventStoreWithXid eventStore = new TwineBallEventStoreWithXid((TwineBallActivationSpecWithXid) activationSpec, this);
        getLogUtils().traceMethodExit(TwineBallEventStoreWithXid.class.getName(), "createEventStore()");
        return eventStore;
    }

    /**
     * @return an instance of WBIResourceAdapterMetadata
     */
    public WBIResourceAdapterMetadata getResourceAdapterMetadata() throws ResourceException {
        return new TwineBallResourceAdapterMetaData(this);
    }
}