/*
 * Licensed Materials - Property of IBM pidnumber (C) Copyright IBM Corporation
 * 2005. All Rights Reserved. US Government Users Restricted Rights- Use,
 * duplication or disclosure restricted by GSA ADP Schedule Contract with IBM
 * Corp.
 */
package com.ibm.j2ca.sample.twineball.emd;

import com.ibm.j2ca.extension.emd.discovery.WBIAdapterTypeImpl;
import com.ibm.j2ca.extension.emd.discovery.WBIMetadataDiscoveryImpl;
import com.ibm.j2ca.sample.twineball.emd.discovery.connection.TwineBallInboundConnectionType;
import com.ibm.j2ca.sample.twineball.emd.discovery.connection.TwineBallOutboundConnectionType;
import commonj.connector.metadata.MetadataException;

/**
 * This defines the AdapterType for TwineBall, along with the inbound and outbound connections.
 */
public class TwineBallAdapterType extends WBIAdapterTypeImpl {
    private static final String ADAPTERTYPE_PROPERTY = "AdapterType";
    public static final String VENDOR = "IBM";
    public static final String VERSION = "6.0.0";
    
    /**
     * Define our outbound and inbound connections for TwineBall.
     * @throws MetadataException
     */
    public TwineBallAdapterType()throws MetadataException{
        super(Constants.RESOURCE_ADAPTER_BEAN_NAME, 2, 1);
        setId(Constants.ADAPTER_NAME);
        setDisplayName(Constants.ADAPTER_NAME);
        setDescription(WBIMetadataDiscoveryImpl.getPropertyDescription(ADAPTERTYPE_PROPERTY));
        setVendor(VENDOR);
        setVersion(VERSION);
        setOutboundConnections();
        setInboundConnections();
    }

    /**
     * Define inbound connections
     * @throws MetadataException
     */
    private void setInboundConnections() throws MetadataException {
        TwineBallInboundConnectionType inConnTypeForRuntime = new TwineBallInboundConnectionType(this);
        addInboundConnectionType(inConnTypeForRuntime);
        inConnTypeForRuntime.setResourceAdapterJavaBean(Constants.RESOURCE_ADAPTER_BEAN_NAME);
        inConnTypeForRuntime.setActivationSpecJavaBean(Constants.ACTIVATION_SPEC);
    }

    /**
     * Define outbound connections
     * @throws MetadataException
     */
    private void setOutboundConnections() throws MetadataException {
        TwineBallOutboundConnectionType outConnTypeForRuntime = new TwineBallOutboundConnectionType(this);
        TwineBallOutboundConnectionType outConnTypeForMetadata = new TwineBallOutboundConnectionType(this);
        outConnTypeForMetadata.setManagedConnectionFactoryJavaBean(Constants.MANAGED_CONNECTION_FACTORY_NAME);
        outConnTypeForRuntime.setManagedConnectionFactoryJavaBean(Constants.MANAGED_CONNECTION_FACTORY_NAME);
        outConnTypeForMetadata.setResourceAdapterJavaBean(Constants.RESOURCE_ADAPTER_BEAN_NAME);
        outConnTypeForRuntime.setResourceAdapterJavaBean(Constants.RESOURCE_ADAPTER_BEAN_NAME);
        outConnTypeForMetadata.setIsSupportedInMetadataService(true);
        outConnTypeForMetadata.setIsSupportedAtRuntime(false);
        outConnTypeForRuntime.setIsSupportedInMetadataService(false);
        outConnTypeForRuntime.setIsSupportedAtRuntime(true);
        addOutboundConnectionType(outConnTypeForMetadata);
        addOutboundConnectionType(outConnTypeForRuntime);
    }
    
}
