/*
 * Licensed Materials - Property of IBM pidnumber (C) Copyright IBM Corporation
 * 2005. All Rights Reserved. US Government Users Restricted Rights- Use,
 * duplication or disclosure restricted by GSA ADP Schedule Contract with IBM
 * Corp.
 */

package com.ibm.j2ca.sample.twineball.emd.description;

import java.util.*;
import java.util.logging.Level;
import javax.xml.namespace.QName;
import com.ibm.j2ca.base.TopLevelVerbs;
import com.ibm.j2ca.extension.emd.description.*;
import com.ibm.j2ca.extension.emd.discovery.WBIMetadataDiscoveryImpl;
import com.ibm.j2ca.extension.emd.discovery.WBIMetadataObjectImpl;
import com.ibm.j2ca.sample.twineball.emd.Constants;
import com.ibm.j2ca.sample.twineball.emd.StringCaseChanger;
import com.ibm.j2ca.sample.twineball.emd.discovery.TwineBallMetadataObject;
import commonj.connector.metadata.MetadataException;
import commonj.connector.metadata.description.SchemaDefinition;
import commonj.connector.metadata.discovery.MetadataObjectResponse;


/**
 * Here we're going to fully describe our data and let the base class generate an XSD for it.
 */
public class TwineBallDataDescription extends WBIDataDescriptionImpl {

    private static final String CLASSNAME = "TwineBallDataDescription"; 

 
    /** 
     * Everything is a string or a child. 
     * @param attrName
     * @return
     * @see com.ibm.j2ca.extension.emd.description.WBIDataDescriptionImpl#getType(java.lang.String)
     */
    public String getType(String attrName) {
        if (getCardinality(attrName) == Constants.N_CARDINALITY) {
            return attrName.toLowerCase() + ":" + StringCaseChanger.toCamelCase(attrName);
        }
        return Constants.XS_STRING;
    }

    /**
     * For any children, prepare schema files for them.
     * @see com.ibm.j2ca.extension.emd.WBIDataDescriptionImpl#prepareChildSchemaFiles()
     */
    public void prepareChildSchemaFiles() throws MetadataException {
        WBIMetadataDiscoveryImpl.getLogUtils().trace(Level.FINER, CLASSNAME, "prepareChildSchemaFiles", "Entering Method");
        MetadataObjectResponse response = getMetadataObject().getChildren(null);
        WBIMetadataDiscoveryImpl.getLogUtils().trace(Level.FINER, CLASSNAME, "prepareChildSchemaFiles", "Number of BUSINESS OBJECTS ===> " + response.getObjectIterator().size());
        for (Iterator i = response.getObjectIterator(); i.hasNext();) {
            TwineBallMetadataObject bo = (TwineBallMetadataObject) i.next();
            TwineBallDataDescription dataDesc = new TwineBallDataDescription();
            dataDesc.setTopLevel(false);
            dataDesc.setMetadataObject(bo);
			dataDesc.setRelativePath(getRelativePath());
            dataDesc.setName(getName().getNamespaceURI(), bo.getBOName());
            WBIMetadataDiscoveryImpl.getLogUtils().trace(Level.FINER, CLASSNAME, "prepareChildSchemaFiles", "Build Child Business Object : " + bo.getBOName());
            if (bo.getType() == WBIMetadataObjectImpl.OBJECT)
                dataDesc.prepareChildSchemaFiles();
            WBIMetadataDiscoveryImpl.getLogUtils().trace(Level.FINER, CLASSNAME, "preparingChildSchemaFiles", "Preparing SchemaFile for " + bo.getDisplayName());
            dataDesc.prepareSchemaFiles();
            SchemaDefinition[] schemaFiles = dataDesc.getSchemaDefinitions();
            for (int j = 0; j < schemaFiles.length; j++) {
                SchemaDefinition definition = schemaFiles[j];
                put(definition.getNamespace(), definition.getLocation(), definition.getContent());
            }
        }
        WBIMetadataDiscoveryImpl.getLogUtils().trace(Level.FINER, CLASSNAME, "prepareChildSchemaFiles", "Exiting Method");
    }

    /**
     *  define the verbs we will support.
     */
    public List getVerbs() {
        ArrayList list = new ArrayList();
        list.add(TopLevelVerbs.CREATE_TLV);
        list.add(TopLevelVerbs.UPDATE_TLV);
        list.add(TopLevelVerbs.DELETE_TLV);
        return list;
    }

    /**
     * Returns WBIMetadata for a given Attribute
     */
    public WBIMetadata getMetadataForAttribute(String attrName) {
        WBIMetadata attributeMetadata = new WBIMetadata();
        attributeMetadata.setSource(Constants.ASI_TARGET_NAMESPACE);
        attributeMetadata.setObjectNameSpace(Constants.ATTR_APPINFO_ASI_TYPE_TAG);
        QName asiNamespace = new QName(Constants.ASI_TARGET_NAMESPACE, Constants.ASI);
        attributeMetadata.setNameSpace(asiNamespace);
        attributeMetadata.setASI(Constants.FIELD_NAME, attrName);
        if (attrName.equalsIgnoreCase(Constants.PRIMARYKEY)) {
            attributeMetadata.setASI(Constants.PRIMARY_KEY, "true");
        }
        return attributeMetadata;
    }

    /**
     * Returns WBIMetadata for a given BusinessObject
     */
    public WBIMetadata getMetadataForBusinessObject() {
        WBIMetadataDiscoveryImpl.getLogUtils().traceMethodEntrance(CLASSNAME, "getMetadataForBusinessObject");
        WBIMetadata bometadata = new WBIMetadata();
        bometadata.setSource(Constants.ASI_TARGET_NAMESPACE);
        QName namespace = new QName(Constants.ASI_TARGET_NAMESPACE, Constants.ASI);
        bometadata.setNameSpace(namespace);
        bometadata.setObjectNameSpace(Constants.BUS_OBJ_APPINFO_ASI_TYPE_TAG);
        bometadata.setASI(Constants.ASI_OBJECTNAME, this.getMetadataObject().getDisplayName());
        WBIMetadataDiscoveryImpl.getLogUtils().traceMethodExit(CLASSNAME, "getMetadataForBusinessObject");
        return bometadata;
    }//End of BO level metadata

    public boolean isContainer() {
        WBIMetadataDiscoveryImpl.getLogUtils().traceMethodEntrance(CLASSNAME, "isContainer");
        boolean retValue = false;
        String objName = getName().getLocalPart();
        if (objName.endsWith("Container") && !objName.equals(getBOName()))
            retValue = true;
        WBIMetadataDiscoveryImpl.getLogUtils().traceMethodExit(CLASSNAME, "isContainer");
        return retValue;
    }

    /**
     * @see com.ibm.j2ca.extension.emd.description.WBIDataDescriptionImpl#getAttributeName(java.lang.String)
     */
    public String getAttributeName(String attrName) {
        return StringCaseChanger.toCamelCase(attrName);
    }

    /**
     * @see com.ibm.j2ca.extension.emd.description.WBIDataDescriptionImpl#getImportNameSpaces()
     */
    public List getImportNameSpaces() throws MetadataException {
        WBIMetadataDiscoveryImpl.getLogUtils().traceMethodEntrance(CLASSNAME, "getImportNameSpaces");
        ArrayList list = new ArrayList();
        ImportedNameSpace namespace = new ImportedNameSpace();
        namespace.setLocation(Constants.TWINEBALLASI_XSD);
        namespace.setNameSpace(Constants.ASI_TARGET_NAMESPACE);
        list.add(namespace);
        WBIMetadataDiscoveryImpl.getLogUtils().traceMethodExit(CLASSNAME, "getImportNameSpaces");
        return list;
    }

    /**
     * @see com.ibm.j2ca.extension.emd.description.WBIDataDescriptionImpl#getNameSpaces()
     */
    public List getNameSpaces() {
        WBIMetadataDiscoveryImpl.getLogUtils().traceMethodEntrance(CLASSNAME, "getNameSpaces");
        ArrayList list = new ArrayList();
        list.add(new QName(Constants.ASI_TARGET_NAMESPACE, Constants.ASI));
        WBIMetadataDiscoveryImpl.getLogUtils().traceMethodExit(CLASSNAME, "getNameSpaces");
        return list;
    }

    public String getASISchemaName() {
        return Constants.ASI_TARGET_NAMESPACE;
    }

    /**
     * @see com.ibm.j2ca.extension.emd.description.WBIDataDescriptionImpl#getCardinality(java.lang.String)
     */
    public String getCardinality(String attrName) {
        TwineBallAttribute definition = (TwineBallAttribute) this.getAttributeList().get(attrName);
        if (definition.isChild)
            return Constants.N_CARDINALITY;
        else
            return "1";
    }

    /**
     * @see com.ibm.j2ca.extension.emd.description.WBIDataDescriptionImpl#getMaxLength(java.lang.String)
     */
    public int getMaxLength(String name) {
        return 0;
    }

    /**
     * @see com.ibm.j2ca.extension.emd.description.WBIDataDescriptionImpl#getRequired(java.lang.String)
     */
    public boolean getRequired(String attrName) {
        return false;
    }

    /** 
     * @return
     * @see com.ibm.j2ca.extension.emd.description.WBIDataDescriptionImpl#getChildList()
     */
    public Iterator getChildList() throws MetadataException{
        return (this.getMetadataObject().getChildren(null)).getObjectIterator();
    }
}