/*
 * Licensed Materials - Property of IBM pidnumber (C) Copyright IBM Corporation
 * 2005. All Rights Reserved. US Government Users Restricted Rights- Use,
 * duplication or disclosure restricted by GSA ADP Schedule Contract with IBM
 * Corp.
 */
package com.ibm.j2ca.sample.twineball.emd.description;

import java.util.ArrayList;

import com.ibm.j2ca.extension.emd.description.EMDConstants;
import com.ibm.j2ca.extension.emd.description.WBIDataDescriptionImpl;
import com.ibm.j2ca.extension.emd.description.WBIInboundFunctionDescriptionImpl;
import com.ibm.j2ca.extension.emd.description.WBIInboundServiceDescriptionImpl;
import com.ibm.j2ca.extension.emd.discovery.WBIMetadataImportConfigurationImpl;
import com.ibm.j2ca.extension.emd.discovery.WBIMetadataSelectionImpl;
import com.ibm.j2ca.extension.emd.discovery.connection.WBIMetadataConnectionImpl;
import com.ibm.j2ca.extension.emd.discovery.properties.WBIMultiValuedPropertyImpl;
import com.ibm.j2ca.extension.emd.discovery.properties.WBISingleValuedPropertyImpl;
import com.ibm.j2ca.sample.twineball.emd.StringCaseChanger;
import com.ibm.j2ca.sample.twineball.emd.discovery.TwineBallMetadataObject;
import com.ibm.j2ca.sample.twineball.emd.discovery.connection.TwineBallConfigurationProperties;
import commonj.connector.metadata.MetadataConfigurationType;
import commonj.connector.metadata.MetadataException;
import commonj.connector.metadata.description.FunctionDescription;
import commonj.connector.metadata.discovery.MetadataImportConfiguration;
import commonj.connector.metadata.discovery.MetadataSelection;
import commonj.connector.metadata.discovery.properties.PropertyGroup;

/**
 * Define our inbound service.   We need to define the interface that will be generated.
 */
public class TwineBallInboundServiceDescription extends WBIInboundServiceDescriptionImpl {
    
    
    /** 
     * Define the functions based on supported objects and operations.
     * @param selection
     * @throws MetadataException
     * @see com.ibm.j2ca.extension.emd.description.WBIInboundServiceDescriptionImpl#setFunctionDescriptions(commonj.connector.metadata.discovery.MetadataSelection)
     */
    public void setFunctionDescriptions(MetadataSelection selection) throws MetadataException {
        MetadataImportConfiguration[] selectedObjects = selection.getSelection();
        PropertyGroup selectionProperties = ((WBIMetadataSelectionImpl) selection).getAppliedSelectionProperties();
        WBIMultiValuedPropertyImpl operationsProperty = (WBIMultiValuedPropertyImpl) selectionProperties.getProperty("Operations");
        String[] operations = operationsProperty.getValuesAsStrings();
        ArrayList functionDescriptions = new ArrayList();
        String location = TwineBallConfigurationProperties.getLocation(selectionProperties);
        
        //iterate through the objects.
        for (int i = 0; i < selectedObjects.length; i++) {
            WBIMetadataImportConfigurationImpl spec = (WBIMetadataImportConfigurationImpl) selectedObjects[i];
            WBIInboundFunctionDescriptionImpl functionDescription;
            TwineBallMetadataObject metadataObj = (TwineBallMetadataObject) spec.getMetadataObject();
            for (int j = 0; j < operations.length; j++) {
                String operation = operations[j];
                functionDescription = new WBIInboundFunctionDescriptionImpl();
                char firstCharacter = operation.charAt(0);
                //convention for displaying the outbound function.
                functionDescription.setName("emit" + Character.toUpperCase(firstCharacter) + operation.substring(1).toLowerCase() + "AfterImage" + StringCaseChanger.toCamelCase(metadataObj.getDisplayName()));
                functionDescription.setEISFunctionName(functionDescription.getName());
                functionDescription.setImportConfiguration(spec);
                TwineBallDataDescription dataDescription = new TwineBallDataDescription();
                dataDescription.setMetadataObject(metadataObj);
                dataDescription.setName(getNameSpace(), StringCaseChanger.toCamelCase(metadataObj.getDisplayName()));
                dataDescription.setRelativePath(location);
                //dataDescription.setName(getNameSpace() + "/" + metadataObj.getBOName().toLowerCase() + "bg", metadataObj.getBOName() + "BG");

                //--CR87212 begins
                WBISingleValuedPropertyImpl property = (WBISingleValuedPropertyImpl) selectionProperties.getProperty(EMDConstants.ARTIFACTS_SUPPORTED);
            	//WBIMetadataDiscoveryImpl.getLogUtils().trace(Level.ALL, "Description", "setupMetadataObject", property.getValue()+"");
            	
            	//Change made for making BG Optional
            	if (property.getValue().equals(MetadataConfigurationType.GENERATED_DATA_BINDING.toString()))
            		dataDescription.setTopLevel(true);
            	else
            		dataDescription.setTopLevel(false);
            	
            	//This is just to force it to be generatedRecord, should be removed
            	//property.setValue(EMDConstants.GENERATED_RECORDS);
            	String namespace = getNameSpace();
            	//Change made for making BG Optional 
            	if (property.getValue().equals(MetadataConfigurationType.GENERATED_DATA_BINDING.toString())) {
                    dataDescription.populateSchemaDefinitions();
            		dataDescription.setName(WBIDataDescriptionImpl.convertNamespaceToUri(namespace + "/" //$NON-NLS-1$
        				+ metadataObj.getBOName().toLowerCase() + "bg" ), metadataObj //$NON-NLS-1$
        				.getBOName() + "BG"); //$NON-NLS-1$
            		dataDescription.setDataBindingGeneratorClassName("com.ibm.j2ca.sample.twineball.emd.runtime.TwineBallDataBindingGenerator");
            		dataDescription.setGenericDataBindingClassName(null);
            		}
            	else if (property.getValue().equals(MetadataConfigurationType.GENERATED_RECORDS.toString())) {
                    dataDescription.populateSchemaDefinitions();
            		dataDescription.setName(WBIDataDescriptionImpl.convertNamespaceToUri(namespace + "/" //$NON-NLS-1$
            				+ metadataObj.getBOName().toLowerCase()), metadataObj //$NON-NLS-1$
            				.getBOName()); //$NON-NLS-1$
            		dataDescription.setDataBindingGeneratorClassName("com.ibm.j2ca.extension.dataexchange.bean.generator.RecordGenerator");
            		dataDescription.setGenericDataBindingClassName(null);
            	} else {
            		//Generic Record Scenario
                    dataDescription.populateSchemaDefinitions();
            		dataDescription.setName(WBIDataDescriptionImpl.convertNamespaceToUri(namespace + "/" //$NON-NLS-1$
            				+ metadataObj.getBOName().toLowerCase()), metadataObj //$NON-NLS-1$
            				.getBOName()); //$NON-NLS-1$
            		dataDescription.setGenericDataBindingClassName("com.ibm.j2ca.sample.twineball.TwineBallStructuredRecord");
            	}
            	//End Change
                //--CR87212 ends
            	WBIMetadataConnectionImpl.getToolContext().getProgressMonitor().setNote("Business object definitions created");
                functionDescription.setInputDataDescription(dataDescription);
                functionDescriptions.add(functionDescription);
            }
        }
        FunctionDescription[] funcArray = new FunctionDescription[functionDescriptions.size()];
        functionDescriptions.toArray(funcArray);
        super.setFunctionDescriptions(funcArray);
    }
}