/*
 * Licensed Materials - Property of IBM pidnumber (C) Copyright IBM Corporation
 * 2005. All Rights Reserved. US Government Users Restricted Rights- Use,
 * duplication or disclosure restricted by GSA ADP Schedule Contract with IBM
 * Corp.
 */
package com.ibm.j2ca.sample.twineball.emd.discovery;

import java.util.logging.Level;

import com.ibm.j2ca.extension.emd.EMDUtil;
import com.ibm.j2ca.extension.emd.discovery.WBIMetadataDiscoveryImpl;
import com.ibm.j2ca.extension.emd.discovery.WBIMetadataSelectionImpl;
import com.ibm.j2ca.extension.emd.discovery.properties.WBIPropertyGroupImpl;
import com.ibm.j2ca.extension.emd.discovery.properties.WBISingleValuedPropertyImpl;
import com.ibm.j2ca.sample.twineball.emd.*;
import commonj.connector.metadata.MetadataException;
import commonj.connector.metadata.MetadataConfigurationType;
import commonj.connector.metadata.discovery.properties.PropertyGroup;

/**
 * This allows us to select the operations we want, as well as specify where the schemas are generated and our maximum result set limit.
 */
public class TwineBallMetadataSelection extends WBIMetadataSelectionImpl {

    public static final String RELATIVEPATH = "RelativePath";
    public static final String MAXRECORDS = "MaxRecords";

    public String METHODNAME = null;
	public String CLASSNAME = "TwineBallMetadataSelection";

    /**
     * Create our properties for import.
     */
    public PropertyGroup createSelectionProperties() {

        WBIPropertyGroupImpl propertyGroup = null;

        METHODNAME = "createSelectionProperties()";

        try {
            propertyGroup = new WBIPropertyGroupImpl(Constants.SELECTION_PROPERTIES);
            propertyGroup.setDisplayName(WBIMetadataDiscoveryImpl.getPropertyName(Constants.SELECTIONPROPERTIES));
            propertyGroup.setDescription(WBIMetadataDiscoveryImpl.getPropertyDescription(Constants.SELECTIONPROPERTIES));            //-CR87212 begins            MetadataConfigurationType[] supportedValues = TwineBallMetadataDiscovery.getMetadataConfiguration();            TwineBallMetadataDiscovery.getLogUtils().trace(Level.ALL, CLASSNAME, METHODNAME, "Getting the Artifact Property");            WBISingleValuedPropertyImpl prop = EMDUtil.getBrokerProperty(supportedValues);            TwineBallMetadataDiscovery.getLogUtils().trace(Level.ALL, CLASSNAME, METHODNAME, "The property values are " + prop.getValue());            propertyGroup.addProperty(prop);            //--CR87212 ends
            //-CR87212 begins
            MetadataConfigurationType[] supportedValues = TwineBallMetadataDiscovery.getMetadataConfiguration();
            TwineBallMetadataDiscovery.getLogUtils().trace(Level.ALL, CLASSNAME, METHODNAME, "Getting the Artifact Property");
            WBISingleValuedPropertyImpl prop = EMDUtil.getBrokerProperty(supportedValues);
            TwineBallMetadataDiscovery.getLogUtils().trace(Level.ALL, CLASSNAME, METHODNAME, "The property values are " + prop.getValue());
            propertyGroup.addProperty(prop);
            //--CR87212 ends
            WBISingleValuedPropertyImpl typeProp = createServiceTypeProperty(propertyGroup);
            ServiceTypeSingleProperty operationProp = createNamespaceProperty(propertyGroup);
    		createMaxRecordProperty(propertyGroup);
    		createRelativePathProperty(propertyGroup);
            //Copy the applied properties to the new instance
            if (this.getAppliedSelectionProperties() != null)
                EMDUtil.copyValues(this.getAppliedSelectionProperties(), propertyGroup);
            typeProp.addPropertyChangeListener(operationProp);
        } catch (MetadataException e) {
            throw new RuntimeException(e);
        }
        return propertyGroup;
    }

    /**
     * @param propertyGroup
     * @return
     * @throws MetadataException
     */
    private WBISingleValuedPropertyImpl createServiceTypeProperty(WBIPropertyGroupImpl propertyGroup) throws MetadataException {
        WBISingleValuedPropertyImpl typeProp = new WBISingleValuedPropertyImpl(Constants.SERVICETYPE, String.class);
        String[] values = { Constants.INBOUND, Constants.OUTBOUND };
        typeProp.setValidValues(values);
        typeProp.setDefaultValue(Constants.OUTBOUND);
        typeProp.setDisplayName(WBIMetadataDiscoveryImpl.getPropertyName(Constants.SERVICETYPE));
        typeProp.setDescription(WBIMetadataDiscoveryImpl.getPropertyDescription(Constants.SERVICETYPE));
        propertyGroup.addProperty(typeProp);
        return typeProp;
    }

    /**
     * @param propertyGroup
     * @return
     * @throws MetadataException
     */
    private ServiceTypeSingleProperty createNamespaceProperty(WBIPropertyGroupImpl propertyGroup) throws MetadataException {
        WBISingleValuedPropertyImpl nameSpaceProp = new WBISingleValuedPropertyImpl(Constants.NAMESPACE, String.class);
        nameSpaceProp.setDefaultValue(Constants.TB_DEFAULT_NAMESPACE);
        propertyGroup.addProperty(nameSpaceProp);
        nameSpaceProp.setDisplayName(WBIMetadataDiscoveryImpl.getPropertyName(Constants.NAMESPACE));
        nameSpaceProp.setDescription(WBIMetadataDiscoveryImpl.getPropertyDescription(Constants.NAMESPACE));
        ServiceTypeSingleProperty operationProp = getOperationPropertyGroup();
        propertyGroup.addProperty(operationProp);
        return operationProp;
    }

    /**
     * @param propertyGroup
     * @throws MetadataException
     */
    private void createRelativePathProperty(WBIPropertyGroupImpl propertyGroup) throws MetadataException {
        WBISingleValuedPropertyImpl location = new WBISingleValuedPropertyImpl(RELATIVEPATH, String.class);
        location.setDisplayName(WBIMetadataDiscoveryImpl.getPropertyName(RELATIVEPATH));
        location.setDescription(WBIMetadataDiscoveryImpl.getPropertyDescription(RELATIVEPATH));
        propertyGroup.addProperty(location);
    }

    /**
     * @param propertyGroup
     * @throws MetadataException
     */
    private void createMaxRecordProperty(WBIPropertyGroupImpl propertyGroup) throws MetadataException {
        MaxRecordSingleProperty maxCount = new MaxRecordSingleProperty(MAXRECORDS, Integer.class);
        maxCount.setDisplayName(WBIMetadataDiscoveryImpl.getPropertyName(MAXRECORDS));
        maxCount.setDescription(WBIMetadataDiscoveryImpl.getPropertyDescription(MAXRECORDS));
        maxCount.setDefaultValue(new Integer("100"));
        propertyGroup.addProperty(maxCount);
    }

    /**
     * @return
     * @throws MetadataException
     */
    private ServiceTypeSingleProperty getOperationPropertyGroup() throws MetadataException {
        ServiceTypeSingleProperty operationProp = new ServiceTypeSingleProperty(Constants.OPERATIONS, String.class);
        String[] operations = TwineBallOperations.getOutboundOperations();
        operationProp.setValidValues(operations);
        operationProp.setDisplayName(WBIMetadataDiscoveryImpl.getPropertyName(Constants.OPERATIONS));
        operationProp.setDescription(WBIMetadataDiscoveryImpl.getPropertyDescription(Constants.OPERATIONS));
        for (int i = 0; i < operations.length; i++) {
            operationProp.addValue(operations[i]);
        }
        return operationProp;
    }
    public static String getNamespace() {
        PropertyGroup pg = getAppliedSelectionProperties();
        if (pg != null) {
            WBISingleValuedPropertyImpl namespaceProp = (WBISingleValuedPropertyImpl) pg.getProperty("NameSpace"); //$NON-NLS-1$
            if (namespaceProp.getValue() != null)
                return (String) namespaceProp.getValue();
        }
        return Constants.TB_DEFAULT_NAMESPACE;
    }

}