/*
 * Licensed Materials - Property of IBM pidnumber (C) Copyright IBM Corporation
 * 2005. All Rights Reserved. US Government Users Restricted Rights- Use,
 * duplication or disclosure restricted by GSA ADP Schedule Contract with IBM
 * Corp.
 */
package com.ibm.j2ca.sample.twineball.emd.discovery;

import java.rmi.RemoteException;
import java.util.*;
import javax.resource.ResourceException;
import com.ibm.j2ca.extension.emd.discovery.*;
import com.ibm.j2ca.extension.emd.discovery.connection.WBIMetadataConnectionImpl;
import com.ibm.j2ca.extension.emd.discovery.connection.WBIOutboundConnectionConfigurationImpl;
import com.ibm.j2ca.extension.emd.discovery.properties.WBIPropertyGroupImpl;
import com.ibm.j2ca.extension.emd.discovery.properties.WBISingleValuedPropertyImpl;
import com.ibm.j2ca.extension.logging.LogUtils;
import com.ibm.j2ca.sample.twineball.emd.*;
import com.ibm.j2ca.sample.twineball.outbound.TwineBallConnection;
import com.ibm.j2ca.sample.twineball.outbound.TwineBallManagedConnection;
import com.ibm.wbia.TwineBall.Server.TwineBallException;
import com.ibm.wbia.TwineBall.Server.TwineBallInterface;
import commonj.connector.metadata.MetadataException;
import commonj.connector.metadata.discovery.*;
import commonj.connector.metadata.discovery.properties.PropertyGroup;

/**
 * This is the metadata tree that will be displayed to the user.
 */
public class TwineBallMetadataTree extends WBIMetadataTreeImpl {
    WBIMetadataConnectionImpl connection;
    private String path = "";
    private static Hashtable treeNodes = new Hashtable();
    private String prefix = "";
    private boolean additionalASI;
    TwineBallConnection cciConnection;
    TwineBallInterface twineBallConnection;
    private LogUtils logUtils;
    
    
    public TwineBallMetadataTree(WBIMetadataConnectionImpl connection, LogUtils logUtils) throws MetadataException {
        super(connection);
        this.connection = connection;
        this.logUtils = logUtils;
        try {
            cciConnection = (TwineBallConnection) connection.getEISConnection();
            twineBallConnection = cciConnection.getEISConnection();
            WBIOutboundConnectionConfigurationImpl conf = (WBIOutboundConnectionConfigurationImpl) connection.getConnectionCofiguration();
            PropertyGroup propertyGroup = conf.getAppliedProperties();
            TwineBallManagedConnection managedConnection = (TwineBallManagedConnection) cciConnection.getManagedConnection();
        } catch (ResourceException e) {
            throw new MetadataException(e.getMessage(), e);
        }
    }

    protected static void addToTree(String location, MetadataObject metaObject) {
        treeNodes.put(location, metaObject);
    }

    /*
     * (non-Javadoc)
     * 
     * @see commonj.connector.metadata.discovery.MetadataTree#createMetaDataSelection()
     */
    public MetadataSelection createMetaDataSelection() {
        return new TwineBallMetadataSelection();
    }

    /*
     * (non-Javadoc)
     * 
     * @see commonj.connector.metadata.discovery.MetadataTree#createFilterProperties()
     */
    public PropertyGroup createFilterProperties() {
        //twineball emd does not support metadata tree filtering. 
        return null;
    }

    /*
     * (non-Javadoc)
     * 
     * @see commonj.connector.metadata.discovery.MetadataTree#getMetaDataObject(java.lang.String,
     *      commonj.connector.metadata.discovery.ToolEnvironmentContext)
     */
    public MetadataObject getMetadataObject(String objectLocationID) {
        return (MetadataObject) treeNodes.get(objectLocationID);
    }

    /*
     * (non-Javadoc)
     * 
     * @see commonj.connector.metadata.discovery.MetadataTree#listMetadataObjects(commonj.connector.metadata.discovery.properties.PropertyGroup,
     *      commonj.connector.metadata.discovery.ToolEnvironmentContext)
     */
    public MetadataObjectResponse listMetadataObjects(PropertyGroup filterParameters) throws MetadataException {
        WBIMetadataObjectResponseImpl response = new WBIMetadataObjectResponseImpl();
        ArrayList objects = getTopLevelObjects();
        response.setObjects(objects);
        return response;
    }

    /**
     * Give us the list of top-level objects in the tree.  We'll later drill down into the children.
     * @return
     * @throws MetadataException
     */
    private ArrayList getTopLevelObjects() throws MetadataException {
        ArrayList objectList = new ArrayList();
        try {
            Vector topLevelObjects = twineBallConnection.getRootObjects();
            objectList = TwineBallMetadataObjectListFactory.getMetadataObjectsForObjectNames(topLevelObjects, twineBallConnection, logUtils);
        } catch (RemoteException e) {
            throw new MetadataException(e.getMessage(),e);
        } catch (TwineBallException e) {
            throw new MetadataException(e.getMessage(),e);
        }
        return objectList;
    }
}