/*
 * Licensed Materials - Property of IBM pidnumber (C) Copyright IBM Corporation
 * 2005. All Rights Reserved. US Government Users Restricted Rights- Use,
 * duplication or disclosure restricted by GSA ADP Schedule Contract with IBM
 * Corp.
 */
package com.ibm.j2ca.sample.twineball.emd.discovery.connection;

import com.ibm.j2ca.extension.emd.EMDUtil;
import com.ibm.j2ca.extension.emd.discovery.WBIMetadataDiscoveryImpl;
import com.ibm.j2ca.extension.emd.discovery.connection.WBIInboundConnectionConfigurationImpl;
import com.ibm.j2ca.extension.emd.discovery.connection.WBIInboundConnectionTypeImpl;
import com.ibm.j2ca.extension.emd.discovery.properties.WBIPropertyGroupImpl;
import com.ibm.j2ca.extension.emd.discovery.properties.WBISingleValuedPropertyImpl;
import com.ibm.j2ca.sample.twineball.TwineBallResourceAdapter;
import com.ibm.j2ca.sample.twineball.emd.discovery.TwineBallMetadataSelection;
import com.ibm.j2ca.sample.twineball.inbound.TwineBallActivationSpecWithXid;
import commonj.connector.metadata.MetadataException;
import commonj.connector.metadata.discovery.properties.PropertyGroup;

/**
 * This is the connection configuration for inbound.
 */
public class TwineBallInboundConnectionConfiguration extends WBIInboundConnectionConfigurationImpl {
    public TwineBallInboundConnectionConfiguration(WBIInboundConnectionTypeImpl connType) throws MetadataException {
        super(connType);
    }

    /*
     * (non-Javadoc)
     * 
     * @see commonj.connector.metadata.discovery.connection.ConnectionConfiguration#getUnifiedProperties()
     */
    public PropertyGroup createUnifiedProperties() {
        WBIPropertyGroupImpl propGroup = null;
        try {
            propGroup = (WBIPropertyGroupImpl) createActivationSpecProperties();
            TwineBallResourceAdapter ra = new TwineBallResourceAdapter();
            WBIPropertyGroupImpl adapterProp = (WBIPropertyGroupImpl) EMDUtil.getPropertyGroup(ra);
	    if(adapterProp != null)
            	propGroup.addProperty(adapterProp);
            if (getAppliedProperties() != null)
                EMDUtil.copyValues(getAppliedProperties(), propGroup);
        } catch (MetadataException e) {
            throw new RuntimeException(e);
        }
        return propGroup;
    }

    public PropertyGroup createActivationSpecProperties() {
        WBIPropertyGroupImpl connProp = null;
        try {
            connProp = (WBIPropertyGroupImpl) EMDUtil.getPropertyGroup(new TwineBallActivationSpecWithXid());
            WBISingleValuedPropertyImpl namespaceProp = (WBISingleValuedPropertyImpl) connProp.getProperty("BONamespace"); //$NON-NLS-1$ 
            namespaceProp.setValue(TwineBallMetadataSelection.getNamespace());
            WBIPropertyGroupImpl credentialPropertyGroup = new WBIPropertyGroupImpl("UserCredentials");
            credentialPropertyGroup.setDisplayName(WBIMetadataDiscoveryImpl.getPropertyName("UserCredentials"));
            credentialPropertyGroup.setDescription(WBIMetadataDiscoveryImpl.getPropertyDescription("UserCredentials"));
            WBISingleValuedPropertyImpl property = new WBISingleValuedPropertyImpl("UserName", String.class);
            property.setRequired(false);
            property.setDisplayName(WBIMetadataDiscoveryImpl.getPropertyName("UserName"));
            property.setDescription(WBIMetadataDiscoveryImpl.getPropertyDescription("UserName"));
            credentialPropertyGroup.addProperty(property);
            property = new WBISingleValuedPropertyImpl("Password", String.class);
            property.setRequired(false);
            property.setSensitive(true); //show it as **** on display
            property.setDisplayName(WBIMetadataDiscoveryImpl.getPropertyName("Password"));
            property.setDescription(WBIMetadataDiscoveryImpl.getPropertyDescription("Password"));
            credentialPropertyGroup.addProperty(property);
            connProp.addProperty(credentialPropertyGroup);
            property = new WBISingleValuedPropertyImpl("Url", String.class);
            property.setRequired(false);
            property.setDisplayName(WBIMetadataDiscoveryImpl.getPropertyName("Url"));
            property.setDescription(WBIMetadataDiscoveryImpl.getPropertyDescription("Url"));
            connProp.addProperty(property);
            property = new WBISingleValuedPropertyImpl("EventTableName", String.class);
            property.setRequired(true);
            property.setDisplayName(WBIMetadataDiscoveryImpl.getPropertyName("eventTableName"));
            property.setDescription(WBIMetadataDiscoveryImpl.getPropertyDescription("eventTableName"));
            property.setValue("WBIA_EVENTS");
            connProp.addProperty(property);
            if (getAppliedProperties() != null)
                EMDUtil.copyValues(getAppliedProperties(), connProp);
        } catch (MetadataException e) {
            throw new RuntimeException(e);
        }
        return connProp;
    }

    
    
    public PropertyGroup createResourceAdapterProperties() {
        TwineBallResourceAdapter ra = new TwineBallResourceAdapter();
        WBIPropertyGroupImpl adapterProp = null;
        try {
            adapterProp = (WBIPropertyGroupImpl) EMDUtil.getPropertyGroup(ra);
            if (getAppliedProperties() != null && adapterProp != null)
                EMDUtil.copyValues(getAppliedProperties(), adapterProp);
        } catch (Exception e) {
            throw new NullPointerException(e.getMessage());
        }
        return adapterProp;
    }
}