/*
 * Licensed Materials - Property of IBM pidnumber (C) Copyright IBM Corporation
 * 2005. All Rights Reserved. US Government Users Restricted Rights- Use,
 * duplication or disclosure restricted by GSA ADP Schedule Contract with IBM
 * Corp.
 */
package com.ibm.j2ca.sample.twineball.emd.discovery.connection;

import com.ibm.j2ca.extension.emd.EMDUtil;
import com.ibm.j2ca.extension.emd.discovery.WBIMetadataDiscoveryImpl;
import com.ibm.j2ca.extension.emd.discovery.connection.WBIOutboundConnectionConfigurationImpl;
import com.ibm.j2ca.extension.emd.discovery.connection.WBIOutboundConnectionTypeImpl;
import com.ibm.j2ca.extension.emd.discovery.properties.WBIPropertyGroupImpl;
import com.ibm.j2ca.extension.emd.discovery.properties.WBISingleValuedPropertyImpl;
import com.ibm.j2ca.sample.twineball.TwineBallResourceAdapter;
import commonj.connector.metadata.MetadataException;
import commonj.connector.metadata.discovery.properties.PropertyGroup;

/**
 * This is our connection configuration for oubound.  We don't need much beyond the base properties.
 */
public class TwineBallOutboundConnectionConfiguration extends WBIOutboundConnectionConfigurationImpl {

    public TwineBallOutboundConnectionConfiguration(WBIOutboundConnectionTypeImpl connType) throws MetadataException {
        super(connType);
    }

    /*
     * (non-Javadoc)
     *
     * @see commonj.connector.metadata.discovery.connection.ConnectionConfiguration#getUnifiedProperties()
     */
    public PropertyGroup createUnifiedProperties() {
        WBIPropertyGroupImpl propGroup = null;

        if (this.getOutboundConnectionType().isSupportedInMetadataService()) {
            try {
                propGroup = TwineBallConfigurationProperties.getTwineBallConfigurationProperties();
            } catch (MetadataException e) {
                throw new RuntimeException(e);
            }
        } else //Create a property group for runtime
        {
            try {
                propGroup = new WBIPropertyGroupImpl("Outbound Properties");
                propGroup.setDisplayName(WBIMetadataDiscoveryImpl.getPropertyName("OutboundProperties"));
                propGroup.setDescription(WBIMetadataDiscoveryImpl.getPropertyDescription("OutboundProperties"));
                propGroup.addProperty(createManagedConnectionFactoryProperties());
		PropertyGroup raProperties = createResourceAdapterProperties();
		if(raProperties != null)
	            propGroup.addProperty(createResourceAdapterProperties());
            } catch (MetadataException e) {
                throw new RuntimeException(e);
            }
        }
        try {
            if (getAppliedProperties() != null)
                EMDUtil.copyValues(getAppliedProperties(), propGroup);
        } catch (MetadataException e) {
            throw new RuntimeException(e);
        }
        return propGroup;
    }

    /**
     * @return
     * @see commonj.connector.metadata.discovery.connection.ConnectionConfiguration#createResourceAdapterProperties()
     */
    public PropertyGroup createResourceAdapterProperties() {
        TwineBallResourceAdapter ra = new TwineBallResourceAdapter();
        WBIPropertyGroupImpl adapterProp = null;
        try {
            adapterProp = ((WBIPropertyGroupImpl) EMDUtil.getPropertyGroup(((WBIOutboundConnectionTypeImpl) this.getOutboundConnectionType()).getResourceAdapterJavaBean()));
            if (getAppliedProperties() != null && adapterProp != null)
                EMDUtil.copyValues(getAppliedProperties(), adapterProp);
        } catch (MetadataException e) {
            throw new RuntimeException(e.getMessage());
        }
        return adapterProp;
    }

    public PropertyGroup createManagedConnectionFactoryProperties() {
        WBIPropertyGroupImpl connProp = null;
        try {
            connProp = ((WBIPropertyGroupImpl) EMDUtil.getPropertyGroup(((WBIOutboundConnectionTypeImpl) this.getOutboundConnectionType()).getManagedConnectionFactoryJavaBean()));
            WBISingleValuedPropertyImpl url = new WBISingleValuedPropertyImpl("Url", String.class);
            url.setRequired(false);
            url.setDisplayName(WBIMetadataDiscoveryImpl.getPropertyName("Url"));
            url.setDescription(WBIMetadataDiscoveryImpl.getPropertyDescription("Url"));
            connProp.addProperty(url);
            WBISingleValuedPropertyImpl schema = new WBISingleValuedPropertyImpl("Schema", String.class);
            schema.setDisplayName(WBIMetadataDiscoveryImpl.getPropertyName("Schema"));
            schema.setDescription(WBIMetadataDiscoveryImpl.getPropertyDescription("Schema"));
            schema.setRequired(false);
            connProp.addProperty(schema);
            if (getAppliedProperties() != null)
                EMDUtil.copyValues(getAppliedProperties(), connProp);
        } catch (MetadataException e) {
            throw new RuntimeException(e.getMessage());
        }
        return connProp;
    }
}