/*
 * Licensed Materials - Property of IBM pidnumber (C) Copyright IBM Corporation
 * 2005. All Rights Reserved. US Government Users Restricted Rights- Use,
 * duplication or disclosure restricted by GSA ADP Schedule Contract with IBM
 * Corp.
 */
package com.ibm.j2ca.sample.twineball.outbound;

import java.rmi.RemoteException;
import java.util.Iterator;
import java.util.Map;
import java.util.Vector;
import java.util.logging.Level;

import javax.resource.NotSupportedException;
import javax.resource.ResourceException;
import javax.resource.cci.InteractionSpec;
import javax.resource.cci.Record;
import javax.resource.spi.CommException;

import com.ibm.despi.InputAccessor;
import com.ibm.despi.InputCursor;
import com.ibm.despi.exception.DESPIException;
import com.ibm.despi.exception.GetFailedException;
import com.ibm.j2ca.base.DataObjectRecord;
import com.ibm.j2ca.base.WBIConnection;
import com.ibm.j2ca.base.WBIInteraction;
import com.ibm.j2ca.base.WBIInteractionSpec;
import com.ibm.j2ca.base.WBIRecord;
import com.ibm.j2ca.base.exceptions.MissingDataException;
import com.ibm.j2ca.base.internal.WPSServiceHelper;
import com.ibm.j2ca.extension.commandpattern.CommandManagerForCursor;
import com.ibm.j2ca.extension.commandpattern.Interpreter;
import com.ibm.j2ca.extension.dataexchange.bean.DEFactoryJavaBean;
import com.ibm.j2ca.extension.dataexchange.sdo.DEFactorySDO;
import com.ibm.j2ca.extension.logging.LogUtilConstants;
import com.ibm.j2ca.extension.logging.LogUtils;
import com.ibm.j2ca.extension.metadata.Property;
import com.ibm.j2ca.extension.metadata.exceptions.InvalidMetadataException;
import com.ibm.j2ca.sample.twineball.ObjectNaming;
import com.ibm.j2ca.sample.twineball.ObjectSerializer;
import com.ibm.j2ca.sample.twineball.TwineBallConstants;
import com.ibm.j2ca.sample.twineball.TwineBallStructuredRecord;
import com.ibm.j2ca.sample.twineball.inbound.TwineBallEventStore;
import com.ibm.j2ca.sample.twineball.outbound.commands.TwineBallBaseCommand;
import com.ibm.j2ca.sample.twineball.outbound.commands.TwineBallCommandFactory;
import com.ibm.wbia.TwineBall.Server.TwineBallException;
import commonj.sdo.DataObject;

public class TwineBallInteraction extends WBIInteraction {

    private CommandManagerForCursor commandManager;
    private Interpreter interpreter;
    private TwineBallConnection connection;
    private LogUtils logger;
    private TwineBallCommandFactory factory;

    /*
     * (non-Javadoc)
     *
     * @see javax.resource.cci.Interaction#execute(javax.resource.cci.InteractionSpec,
     *      javax.resource.cci.Record, javax.resource.cci.Record)
     */
    public boolean execute(InteractionSpec ispec, Record inRecord, Record outputRecord) throws ResourceException {
    	logger.traceMethodEntrance(TwineBallInteraction.class.getName(), "execute()");
    	TwineBallStructuredRecord outRecord = (TwineBallStructuredRecord)outputRecord;
        try{
        	TwineBallStructuredRecord tbInputRecord = getTBInputRecord(inRecord);
            tbInputRecord.setTwineBallConnection(connection.getEISConnection());
        	commandManager.setDEInterface(tbInputRecord.getDataExchangeFactory());
        	commandManager.setNamespace(TwineBallConstants.METADATA_NAMESPACE);
	        String functionName = ((WBIInteractionSpec) ispec).getFunctionName();
	        traceFine("Function name is: " + functionName);

	        WBIInteractionSpec interactionSpec = (WBIInteractionSpec)ispec;
	        factory.setMaxRecords(interactionSpec.getMaxRecords());
	        tbInputRecord.setTwineBallConnection(connection.getEISConnection());
	        TwineBallBaseCommand topLevelCommand = (TwineBallBaseCommand)commandManager.produceCommands(tbInputRecord, functionName);
	        traceFine("produced commands - top level: " + topLevelCommand.getClass().getName());

	        try {
	            interpreter.execute(topLevelCommand);
	        } catch (ResourceException e) {
	            log(e);
	            throw e;
	        }
	        autoCommit();		
	        
	        if (functionName.equalsIgnoreCase("Retrieve") || functionName.equalsIgnoreCase("Update")) {
	        	setupRetrieve(outRecord, tbInputRecord);
	        }else if (functionName.equalsIgnoreCase("RetrieveAll")){
	        	ObjectSerializer serializer = new ObjectSerializer( new ObjectNaming());
	        	Vector searchPairs = serializer.toNameValueVector(tbInputRecord.getMetadata(),tbInputRecord.getTopLevelCursor());
	        	outRecord.setEISRepresentation(searchPairs);
	        }else if (functionName.equalsIgnoreCase("Create")){
	        	setupRetrieveForCreate(outRecord, topLevelCommand, tbInputRecord);
	        }else{
	        	outRecord.setEISRepresentation(topLevelCommand.getEisRepresentation());
	        }
	        if (functionName.equalsIgnoreCase("Delete")){
	        	outRecord.setOperationName("OutboundDelete");
	        }else if (functionName.equalsIgnoreCase("RetrieveAll")){
	        	outRecord.setOperationName("RetrieveAll");

	        }else{
	        	outRecord.setOperationName(WBIInteractionSpec.RETRIEVE_OP);
	        }
	        outRecord.setTwineBallConnection(connection.getEISConnection());
	        logger.traceMethodExit(TwineBallInteraction.class.getName(), "execute()");
        }catch(Exception e){
        	if (e instanceof ResourceException){
        		throw (ResourceException)e;
        	} else{
                throw new ResourceException(e);
        	}
        }
        return true;
    }

    /*
     * (non-Javadoc)
     *
     * @see javax.resource.cci.Interaction#execute(javax.resource.cci.InteractionSpec,
     *      javax.resource.cci.Record)
     */
    public Record execute(InteractionSpec ispec, Record inRecord) throws ResourceException {
        Record outRecord = new TwineBallStructuredRecord();
        execute(ispec,inRecord,outRecord);
        return outRecord;
    }
    
	private void setupRetrieveForCreate(TwineBallStructuredRecord outRecord, TwineBallBaseCommand createCommand, TwineBallStructuredRecord inputRecord) throws InvalidMetadataException, DESPIException, GetFailedException {
		String primaryKey = createCommand.getKey();
		outRecord.setMetadata(inputRecord.getMetadata());
		Map annotations = inputRecord.getMetadata().getAnnotations(TwineBallConstants.METADATA_NAMESPACE);
		String objectName = (String)annotations.get("ObjectName");
		outRecord.setPrimaryKey(primaryKey);
		outRecord.setObjectName(objectName);
	}

	private void setupRetrieve(TwineBallStructuredRecord outRecord, TwineBallStructuredRecord inputRecord) throws InvalidMetadataException, DESPIException, GetFailedException, MissingDataException {
		outRecord.setEISRepresentation(inputRecord.getEISRepresentation());
		outRecord.setMetadata(inputRecord.getMetadata());
		Map annotations = inputRecord.getMetadata().getAnnotations(TwineBallConstants.METADATA_NAMESPACE);
		String objectName = (String)annotations.get("ObjectName");
		InputCursor cursor = (InputCursor)inputRecord.getTopLevelCursor();
		cursor.getNext();
		Iterator properties = inputRecord.getMetadata().getPropertyIterator();
		String keyProp = "Primarykey";
		while (properties.hasNext()) {
		    Property property = (Property) properties.next();
		    if(property.getName().equalsIgnoreCase("primarykey")) {
		        keyProp = property.getName();
		        break;
		    }
		}
		InputAccessor accessor = (InputAccessor)cursor.getAccessor(keyProp);
		if (accessor == null || accessor.isNull() || !accessor.isSet() || accessor.getString().length() == 0)
		{
    		throw new MissingDataException(outRecord.getTopLevelCursor());
		}
		outRecord.setPrimaryKey(accessor.getString());
		outRecord.setObjectName(objectName);
	}

	private TwineBallStructuredRecord getTBInputRecord(Record inRecord) throws DESPIException, InvalidMetadataException {
		TwineBallStructuredRecord tbInputRecord = null;

		if (inRecord instanceof TwineBallStructuredRecord){
			tbInputRecord = (TwineBallStructuredRecord)inRecord;
	        logger.trace(Level.FINEST,TwineBallInteraction.class.getName(),"execute()","Dumping the input structured record");
	        logger.traceRecord(tbInputRecord,TwineBallInteraction.class.getName(), "execute()");
		}else if (inRecord instanceof DataObjectRecord || inRecord instanceof WBIRecord){
			DataObjectRecord record = (DataObjectRecord)inRecord;
			tbInputRecord = new TwineBallStructuredRecord();
			DEFactorySDO binding = new DEFactorySDO();
			DataObject object = record.getDataObject();
			//Type type = TypeFactory.getType(object);
			commonj.sdo.Property prop = WPSServiceHelper.getRootBusinessObjectProperty(object.getType());
			DataObject inputObject = object.getDataObject(prop);
			binding.setBoundObject(inputObject);
			tbInputRecord.initializeInput(binding, inputObject);
		}else{
			tbInputRecord = new TwineBallStructuredRecord();
			DEFactoryJavaBean binding = new DEFactoryJavaBean();
			binding.setBoundObject(inRecord);
			tbInputRecord.initializeInput(binding, inRecord);
		}
		return tbInputRecord;
	}

    /**
     * @param message
     */
    private void traceFine(String message) {
        logger.trace(Level.FINE, TwineBallInteraction.class.getName(), "execute()", message);
    }

    private void log(ResourceException e) throws ResourceException {
        logger.trace(Level.SEVERE, TwineBallInteraction.class.getName(), "execute()", "Caught exception in outbound processing: " + e.getClass().getName() + ":" + e.getMessage(), e);
        if (e instanceof CommException) {
            ((TwineBallManagedConnection) connection.getManagedConnection()).invalidateThisConnection(e);
            logger.log(Level.SEVERE, LogUtilConstants.ADAPTER_RBUNDLE, TwineBallInteraction.class.getName(), "execute()", "1004", new Object[] {});
        } else {
            logger.log(Level.SEVERE, LogUtilConstants.ADAPTER_RBUNDLE, TwineBallInteraction.class.getName(), "execute()", "1010", new Object[] { e.getClass().getName() });
        }
    }

    /**
     * @param connection
     */
    public TwineBallInteraction(WBIConnection connection) throws ResourceException {
        super(connection);
        logger = this.getLogUtils();
        this.connection = (TwineBallConnection) connection;
        interpreter = new Interpreter(this.getLogUtils());
        //TwineBallResourceAdapter resourceAdapter = this.connection.getResourceAdapter();
        ObjectNaming objectNaming = new ObjectNaming();
        factory = new TwineBallCommandFactory(objectNaming);
        commandManager = new CommandManagerForCursor(factory, this.connection.getEISConnection(), this.getLogUtils());
    }

    public void autoCommit() throws ResourceException {
        if (connection.getAutoCommit()) {
            this.getLogUtils().trace(Level.FINER, TwineBallConnection.class.getName(), "autoCommit()", "attempting auto-commit");
            TwineBallManagedConnection twineBallConnection = (TwineBallManagedConnection) connection.getManagedConnection();
            try {
                twineBallConnection.getEISConnection().commit();
            } catch (RemoteException e) {
                this.getLogUtils().log(Level.SEVERE, LogUtilConstants.ADAPTER_RBUNDLE, TwineBallEventStore.class.getName(), "autoCommit()", "1004", new Object[] {});
                throw new CommException(e);
            } catch (TwineBallException e) {
                throw new ResourceException(e);
            }
        }
    }
}