/*
 * Licensed Materials - Property of IBM pidnumber (C) Copyright IBM Corporation
 * 2005. All Rights Reserved. US Government Users Restricted Rights- Use,
 * duplication or disclosure restricted by GSA ADP Schedule Contract with IBM
 * Corp.
 */
package com.ibm.j2ca.sample.twineball.outbound;

import java.rmi.RemoteException;
import java.util.logging.Level;
import javax.resource.ResourceException;
import javax.resource.spi.CommException;
import com.ibm.j2ca.extension.logging.LogUtilConstants;
import com.ibm.j2ca.extension.logging.LogUtils;
import com.ibm.j2ca.sample.twineball.inbound.TwineBallEventStore;
import com.ibm.wbia.TwineBall.Server.TwineBallException;
import com.ibm.wbia.TwineBall.Server.TwineBallInterface;

public class TwineBallLocalTransaction implements javax.resource.spi.LocalTransaction {
    TwineBallManagedConnection thisConnection;
    LogUtils logUtils;

    TwineBallLocalTransaction(TwineBallManagedConnection thisConnection) {
        this.thisConnection = thisConnection;
        logUtils = thisConnection.getLogUtils();
    }

    /**
     * @see javax.resource.cci.LocalTransaction#begin()
     */
    public void begin() throws ResourceException {
        //we don't have to begin.
    }

    public void commit() throws ResourceException {
        logUtils.trace(Level.FINER, TwineBallLocalTransaction.class.getName(), "commit()", "Commit");
        TwineBallInterface currentApplicationConnection = thisConnection.getEISConnection();
        try {
            currentApplicationConnection.commit();
        } catch (TwineBallException twineBallException) {
            logUtils.log(Level.SEVERE, LogUtilConstants.ADAPTER_RBUNDLE, TwineBallEventStore.class.getName(), "commit()", "1011", new Object[] {});
            throw new ResourceException(twineBallException);
        } catch (RemoteException remoteException) {
            logUtils.log(Level.SEVERE, LogUtilConstants.ADAPTER_RBUNDLE, TwineBallEventStore.class.getName(), "commit()", "1011", new Object[] {});
            thisConnection.invalidateThisConnection(remoteException);
            throw new CommException(remoteException);
        }
    }

    /**
     * @see javax.resource.cci.LocalTransaction#rollback()
     */
    public void rollback() throws ResourceException {
        logUtils.trace(Level.FINER, TwineBallLocalTransaction.class.getName(), "rollback()", "Rollback");
        TwineBallInterface currentApplicationConnection = thisConnection.getEISConnection();
        try {
            currentApplicationConnection.rollback();
        } catch (TwineBallException twineBallException) {
            logUtils.log(Level.SEVERE, LogUtilConstants.ADAPTER_RBUNDLE, TwineBallEventStore.class.getName(), "commit()", "1012", new Object[] {});
            throw new ResourceException(twineBallException);
        } catch (RemoteException remoteException) {
            logUtils.log(Level.SEVERE, LogUtilConstants.ADAPTER_RBUNDLE, TwineBallEventStore.class.getName(), "commit()", "1012", new Object[] {});
            thisConnection.invalidateThisConnection(remoteException);
            throw new CommException(remoteException);
        }
    }
}