/*
 * Licensed Materials - Property of IBM
 * pidnumber
 * (C) Copyright IBM Corporation 2005. All Rights Reserved.
 * US Government Users Restricted Rights- Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 */
package com.ibm.j2ca.sample.twineball.outbound;

import java.rmi.RemoteException;
import java.util.logging.Level;
import javax.resource.ResourceException;
import javax.resource.spi.CommException;
import javax.resource.spi.ManagedConnectionMetaData;
import javax.resource.spi.ResourceAdapter;
import javax.resource.spi.security.PasswordCredential;
import javax.security.auth.Subject;
import javax.transaction.xa.XAResource;
import com.ibm.j2ca.base.WBIConnectionRequestInfo;
import com.ibm.j2ca.base.WBILocalTransactionWrapper;
import com.ibm.j2ca.base.WBIManagedConnection;
import com.ibm.j2ca.base.WBIManagedConnectionFactory;
import com.ibm.j2ca.base.WBIManagedConnectionMetaData;
import com.ibm.j2ca.base.WBIXAResourceWrapper;
import com.ibm.j2ca.extension.logging.LogUtilConstants;
import com.ibm.j2ca.extension.logging.LogUtils;
import com.ibm.wbia.TwineBall.Server.TwineBallException;
import com.ibm.wbia.TwineBall.Server.TwineBallInterface;

/**
 * @author Travis Nelson
 *  
 */
public class TwineBallManagedConnection extends WBIManagedConnection {
    private TwineBallInterface twineballConnection;
    private String userID;
    private String password;
    private String schema;
    private String URL;
    private ResourceAdapter resourceAdapter;
    private LogUtils logUtils;

    /**
     * @param mcf -
     *            the managed connection factory
     * @param subject -
     *            a security subject
     * @param connRequestInfo -
     *            connection request information. May contain credentials
     * @throws ResourceException
     */
    public TwineBallManagedConnection(WBIManagedConnectionFactory managedConnectionFactory, Subject subject, WBIConnectionRequestInfo connRequestInfo, ResourceAdapter resourceAdapter)
            throws ResourceException {
        super((WBIManagedConnectionFactory) managedConnectionFactory, subject, (WBIConnectionRequestInfo) connRequestInfo);
        this.resourceAdapter = resourceAdapter;
        this.logUtils = getLogUtils();
        logUtils.trace(Level.FINER, TwineBallManagedConnection.class.getName(), "Constructor", "creating managed connection");
        setURL(managedConnectionFactory);
        logOn(this.getPasswordCredential());   
    }

    /**
     * @param pc
     * @param factory
     * @throws ResourceException
     * @throws CommException
     */
    private void logOn(PasswordCredential pc) throws ResourceException, CommException {
        try{
            logUtils.trace(Level.FINE, TwineBallManagedConnection.class.getName(), "logOn()", "getting new TwineBall connection ");
            com.ibm.wbia.TwineBall.Server.TwineBallConnectionFactory factory = new com.ibm.wbia.TwineBall.Server.TwineBallConnectionFactory();
            twineballConnection = factory.getConnection(pc.getUserName(), new String(pc.getPassword()), schema, URL);
            twineballConnection.enableTracing(true);
        } catch (RemoteException e) {
            this.invalidateThisConnection(e);
            logUtils.log(Level.SEVERE, LogUtilConstants.ADAPTER_RBUNDLE, TwineBallManagedConnection.class.getName(), "Constructor", "1005", new Object[] {});
            throw new CommException(e);
        } catch (TwineBallException e) {
            throw new ResourceException(e);
        }
    }

    private void setURL(WBIManagedConnectionFactory managedConnectionFactory) {
        TwineBallManagedConnectionFactory factory = (TwineBallManagedConnectionFactory) managedConnectionFactory;
        URL = factory.getUrl();
        schema = factory.getSchema();
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.resource.spi.ManagedConnection#getMetaData()
     */
    public ManagedConnectionMetaData getMetaData() throws ResourceException {
       return new WBIManagedConnectionMetaData("TwineBall","0.2",200,userID);
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.resource.spi.ManagedConnection#destroy()
     */
    public void destroy() throws ResourceException {
        logUtils.trace(Level.FINE, TwineBallManagedConnection.class.getName(), "destroy()", "connection destroyed");
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.j2ca.WBIManagedConnection#getWBIConnection(javax.resource.spi.security.PasswordCredential,
     *      boolean)
     */
    public Object getWBIConnection(PasswordCredential pc, boolean reauthenticate) throws ResourceException {
        logUtils.trace(Level.FINE, TwineBallManagedConnection.class.getName(), "getWBIConnection()", "Attempting to obtain connection for user: " + pc.getUserName());
        return new TwineBallConnection(this);
    }

    public TwineBallInterface getEISConnection() {
        return twineballConnection;
    }

    /**
     * @return Returns the password.
     */
    public String getPassword() {
        return password;
    }

    /**
     * @param password
     *            The password to set.
     */
    void setPassword(String password) {
        this.password = password;
    }

    /**
     * @return Returns the schema.
     */
     public String getSchema() {
        return schema;
    }

    /**
     * @param schema
     *            The schema to set.
     */
   void setSchema(String schema) {
        this.schema = schema;
    }

    /**
     * @return Returns the twineballConnection.
     */
    public TwineBallInterface getTwineballConnection() {
        return twineballConnection;
    }

    /**
     * @param twineballConnection
     *            The twineballConnection to set.
     */
    void setTwineballConnection(TwineBallInterface twineballConnection) {
        this.twineballConnection = twineballConnection;
    }

    /**
     * @return Returns the uRL.
     */
    public String getURL() {
        return URL;
    }

    /**
     * @param url
     *            The uRL to set.
     */
    void setURL(String url) {
        URL = url;
    }

    /**
     * @return Returns the userID.
     */
    public String getUserID() {
        return userID;
    }

    /**
     * @param userID
     *            The userID to set.
     */
    void setUserID(String userID) {
        this.userID = userID;
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.resource.spi.ManagedConnection#getLocalTransaction()
     */
    public javax.resource.spi.LocalTransaction getLocalTransaction() throws ResourceException {
        TwineBallLocalTransaction transaction = new TwineBallLocalTransaction(this);
        return new WBILocalTransactionWrapper(transaction, this);
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.resource.spi.ManagedConnection#getXAResource()
     */
    public XAResource getXAResource() throws ResourceException {
        return new WBIXAResourceWrapper(new TwineBallXATransaction(twineballConnection), this);
    }

    /**
     * @return Returns the resourceAdapter.
     */
    public ResourceAdapter getResourceAdapter() {
        return resourceAdapter;
    }

    public void invalidateThisConnection(Exception e) throws ResourceException {
        logUtils.trace(Level.FINE, TwineBallManagedConnection.class.getName(), "invalidateThisConnection()", e.getClass().getName() + ":" + e.getMessage() ,e );
        this.fireErrorOccurred(e);
    }
}