/*
 * Licensed Materials - Property of IBM pidnumber (C) Copyright IBM Corporation
 * 2005. All Rights Reserved. US Government Users Restricted Rights- Use,
 * duplication or disclosure restricted by GSA ADP Schedule Contract with IBM
 * Corp.
 */
package com.ibm.j2ca.sample.twineball.outbound;

import java.rmi.RemoteException;
import javax.transaction.xa.XAException;
import javax.transaction.xa.XAResource;
import javax.transaction.xa.Xid;
import com.ibm.wbia.TwineBall.Server.TwineBallXA;
import com.ibm.wbia.TwineBall.Server.TwineBallXid;

public class TwineBallXATransaction implements XAResource {
    TwineBallXA xaResource;

    public TwineBallXATransaction(TwineBallXA xaResource) {
        this.xaResource = xaResource;
    }

    public void commit(Xid xid, boolean onePhase) throws XAException {
        try {
            xaResource.commit(new TwineBallXid(xid), onePhase);
        } catch (RemoteException e) {
            throw new XAException(e.getMessage());
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.transaction.xa.XAResource#end(javax.transaction.xa.Xid, int)
     */
    public void end(Xid xid, int flags) throws XAException {
        try {
            xaResource.end(new TwineBallXid(xid), flags);
        } catch (RemoteException e) {
            throw new XAException(e.getMessage());
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.transaction.xa.XAResource#forget(javax.transaction.xa.Xid)
     */
    public void forget(Xid xid) throws XAException {
        try {
            xaResource.forget(new TwineBallXid(xid));
        } catch (RemoteException e) {
            throw new XAException(e.getMessage());
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.transaction.xa.XAResource#getTransactionTimeout()
     */
    public int getTransactionTimeout() throws XAException {
        int timeout = 0;
        try {
            timeout = xaResource.getTransactionTimeout();
        } catch (RemoteException e) {
            throw new XAException(e.getMessage());
        }
        return timeout;
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.transaction.xa.XAResource#isSameRM(javax.transaction.xa.XAResource)
     */
    public boolean isSameRM(XAResource xares) throws XAException {
        boolean isSameRM = false;
        try {
            isSameRM = xaResource.isSameRM(xares);
        } catch (RemoteException e) {
            throw new XAException(e.getMessage());
        }
        return isSameRM;
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.transaction.xa.XAResource#prepare(javax.transaction.xa.Xid)
     */
    public int prepare(Xid xid) throws XAException {
        int prepare = 0;
        try {
            prepare = xaResource.prepare(new TwineBallXid(xid));
        } catch (RemoteException e) {
            throw new XAException(e.getMessage());
        }
        return prepare;
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.transaction.xa.XAResource#recover(int)
     */
    public Xid[] recover(int flag) throws XAException {
        Xid[] recover = null;
        try {
            recover = xaResource.recover(flag);
        } catch (RemoteException e) {
            throw new XAException(e.getMessage());
        }
        return recover;
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.transaction.xa.XAResource#rollback(javax.transaction.xa.Xid)
     */
    public void rollback(Xid xid) throws XAException {
        try {
            xaResource.rollback(new TwineBallXid(xid));
        } catch (RemoteException e) {
            throw new XAException(e.getMessage());
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.transaction.xa.XAResource#setTransactionTimeout(int)
     */
    public boolean setTransactionTimeout(int seconds) throws XAException {
        boolean setTransactionTimeout = false;
        try {
            setTransactionTimeout = xaResource.setTransactionTimeout(seconds);
        } catch (RemoteException e) {
            throw new XAException(e.getMessage());
        }
        return setTransactionTimeout;
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.transaction.xa.XAResource#start(javax.transaction.xa.Xid, int)
     */
    public void start(Xid xid, int flags) throws XAException {
        try {
            xaResource.start(new TwineBallXid(xid), flags);
        } catch (RemoteException e) {
            throw new XAException(e.getMessage());
        }
    }
}