/*
 * Licensed Materials - Property of IBM pidnumber (C) Copyright IBM Corporation
 * 2005. All Rights Reserved. US Government Users Restricted Rights- Use,
 * duplication or disclosure restricted by GSA ADP Schedule Contract with IBM
 * Corp.
 */
package com.ibm.j2ca.sample.twineball.outbound.commands;

import java.util.HashMap;
import java.util.logging.Level;
import javax.resource.ResourceException;
import com.ibm.j2ca.extension.commandpattern.CommandForCursor;
import com.ibm.j2ca.extension.commandpattern.CommandFactoryForCursor;
import com.ibm.j2ca.extension.commandpattern.NodeLevelOperations;
import com.ibm.j2ca.extension.metadata.Type;
import com.ibm.j2ca.sample.twineball.ObjectNaming;
import com.ibm.j2ca.sample.twineball.ObjectSerializer;
import com.ibm.j2ca.extension.metadata.Type;

public class TwineBallCommandFactory implements CommandFactoryForCursor {

    public CommandForCursor createCommand(String nodeLevelOperation) throws ResourceException {
		// TODO Auto-generated method stub
		return createCommand(nodeLevelOperation,null);
	}

	/*
     * (non-Javadoc)
     *
     * @see com.ibm.j2c.commandpattern.CommandFactory#isOOType()
     */
    public boolean isOOType() {
        return true;
    }

    ObjectNaming objectNaming;
    ObjectSerializer objectSerializer;
    HashMap functionBindings;
    int maxRecords;

    public TwineBallCommandFactory(ObjectNaming objectNaming) {
        this.objectNaming = objectNaming;
        objectSerializer = new ObjectSerializer(objectNaming);
    }

    public CommandForCursor createCommand(String functionName, Type metadata) throws ResourceException {

			TwineBallBaseCommand command = null;

			try {

				if (functionName.equals(NodeLevelOperations.CREATE_NODE)) {
					command = new TwineBallCreateCommand();
				} else if (functionName.equals(NodeLevelOperations.DELETE_NODE)) {
					command = new TwineBallDeleteCommand();
				} else if (functionName.equals(NodeLevelOperations.UPDATE_NODE)) {
					command = new TwineBallUpdateCommand();
				} else if (functionName.equals(NodeLevelOperations.RETRIEVE_STRUCTURE)) {
					command = new TwineBallRetrieveCommand();
				} else if (functionName.equals(NodeLevelOperations.RETRIEVE_ALL)) {
					command = new TwineBallRetrieveAllCommand();
				} else {
					command = new TwineBallBaseCommand();
				}
				command.setObjectSerializer(objectSerializer);
				command.setObjectNaming(objectNaming);
				command.setMaxRecords(maxRecords);
				command.setMetadata(metadata);

				if (functionName == NodeLevelOperations.DELETE_NODE) {
					command.setExecutionOrder(CommandForCursor.BEFORE_PARENT);
				} else {
					command.setExecutionOrder(CommandForCursor.AFTER_PARENT);
				}

	        //objectNaming.getLogger().trace(Level.FINEST, TwineBallCommandFactory.class.getName(), "createCommand()", "created command: " + command.getClass().getName());
	        }catch (Exception e) {
	            throw new ResourceException(e);
	        }
        return command;
    }

    /**
     * @return Returns the maxRecords.
     */
    public int getMaxRecords() {
        return maxRecords;
    }

    /**
     * @param maxRecords
     *            The maxRecords to set.
     */
    public void setMaxRecords(int maxRecords) {
        this.maxRecords = maxRecords;
    }
}