/*
 * IBM Confidential OCO Source Materials ADAPTER-PID(s) (C) Copyright IBM
 * Corporation 2004 The source code for this program is not published or
 * otherwise divested of its trade secrets, irrespective of what has been
 * deposited with the U.S. Copyright Office.
 */

package com.ibm.wbia.TwineBall.Server;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Properties;

public class DatabaseCreation {

    Connection connection;

    Statement statement;

    public void createDatabase(Properties properties) throws SQLException, ClassNotFoundException, TwineBallException {
		connection = new DatabaseConnection().getJDBCConnection(properties);
        connection.setAutoCommit(true);
        statement = connection.createStatement();
        if (!testExistence("CUSTOMER")) {
            createBusinessSchema();
            createWorkflows();
        }
    }

    public void createWorkflows() throws SQLException {
        executeIgnoreExceptions("DROP TABLE TWINEBALL.WBIA_EVENTS");
        executeIgnoreExceptions("DROP TRIGGER TWINEBALL.CUSTOMER_CREATE");
        executeIgnoreExceptions("DROP TRIGGER TWINEBALL.CUSTOMER_UPDATE");
        executeIgnoreExceptions("DROP TRIGGER TWINEBALL.CUSTOMER_DELETE");
        executeSQL("CREATE TABLE TWINEBALL.WBIA_EVENTS (" + "PRIMARYKEY		integer not null primary key generated ALWAYS as identity,"
                + "CONNECTOR_ID	varchar(40)," + "OBJECT_KEY		integer not null," + "OBJECT_NAME		varchar(40)	not null," + "OBJECT_VERB		varchar(40)	not null," + "EVENT_PRIORITY	varchar(2) not null,"
                + "EVENT_TIME		timestamp default current timestamp," + "EVENT_STATUS	varchar(2) NOT NULL DEFAULT '0'," + "EVENT_COMMENT	varchar(100)," + "TRIGGERING_USER	varchar(20), XID varchar(256))");

        executeSQL("CREATE TRIGGER TWINEBALL.CUSTOMER_CREATE " +
        		" AFTER  INSERT ON TWINEBALL.CUSTOMER  REFERENCING  NEW AS N "+
        		" FOR EACH ROW MODE DB2SQL " + 
        		" INSERT INTO TWINEBALL.WBIA_EVENTS " +
				" (connector_id, object_key, object_name, object_verb, event_priority, event_status, event_comment, triggering_user) " +
        		" VALUES " + 
				" (\'TwineBall\', N.PRIMARYKEY, \'Customer\', \'Create\', \'1\', \'0\', \'Triggered by Customer CREATE\', USER)" );
        executeSQL("CREATE TRIGGER TWINEBALL.CUSTOMER_UPDATE " +
        		" AFTER  UPDATE ON TWINEBALL.CUSTOMER  REFERENCING  NEW AS N "+
        		" FOR EACH ROW MODE DB2SQL " + 
        		" INSERT INTO TWINEBALL.WBIA_EVENTS " +
				" (connector_id, object_key, object_name, object_verb, event_priority, event_status, event_comment, triggering_user) " +
        		" VALUES (\'TwineBall\', N.PRIMARYKEY, \'Customer\', \'Update\', \'1\', \'0\', \'Triggered by Customer UPDATE\', USER)" );
        executeSQL("CREATE TRIGGER TWINEBALL.CUSTOMER_DELETE " +
        		" AFTER  DELETE ON TWINEBALL.CUSTOMER  REFERENCING  OLD AS O "+
        		" FOR EACH ROW MODE DB2SQL " + 
        		" INSERT INTO TWINEBALL.WBIA_EVENTS" +
				" (connector_id, object_key, object_name, object_verb, event_priority, event_status, event_comment, triggering_user) " +
        		" VALUES " +
				" (\'TwineBall\', O.PRIMARYKEY, \'Customer\', \'Delete\', \'1\', \'0\', \'Triggered by Customer DELETE\', USER)" );
                                
    }

    public void createBusinessSchema() throws SQLException {
        executeIgnoreExceptions("DROP TABLE TWINEBALL.PARENTCHILD");
        executeIgnoreExceptions("DROP TABLE TWINEBALL.CUSTOMER");
        executeIgnoreExceptions("DROP TABLE TWINEBALL.ADDRESS");
        executeIgnoreExceptions("DROP TABLE TWINEBALL.CUSTOMERORDER");
        executeIgnoreExceptions("DROP TABLE TWINEBALL.ORDERITEM");
        
        executeSQL("CREATE TABLE TWINEBALL.PARENTCHILD (" +
        		"PARENT      varchar(35) not null, " + 
				"CHILD		varchar(35) not null)");
        
        executeSQL("CREATE TABLE TWINEBALL.CUSTOMER (" +
        		"PRIMARYKEY		integer not null primary key GENERATED ALWAYS AS IDENTITY, " +
				"CUSTOMERTYPE	varchar(20), " +
				"TITLE			varchar(10), " +
				"FIRSTNAME		varchar(20), " +
				"LASTNAME		varchar(20), " +
				"MIDDLEINITIAL	char(1), " +
				"SEX				char(1), " +
				"PRIMARYPHONE	varchar(20), " +
				"SECONDARYPHONE	varchar(20))");
        
        executeSQL("CREATE TABLE TWINEBALL.ADDRESS ( " +
        		"PRIMARYKEY			integer not null primary key GENERATED ALWAYS AS IDENTITY, " +
				"ADDRESSTYPE			varchar(20), " +
				"ADDRESSLINE1		varchar(30), " +
				"ADDRESSLINE2		varchar(30), " +
				"CITY				varchar(20), " +
				"STATE				varchar(20), " +
				"COUNTRY				varchar(20), " +
				"ZIPCODE				varchar(10), " +
				"PARENTKEY			integer, " +
				"PARENTTABLE			varchar(20))");

        executeSQL("insert into TWINEBALL.PARENTCHILD(PARENT, CHILD) VALUES('CUSTOMER', 'ADDRESS')");
    }

    private void executeIgnoreExceptions(String statementString) {
        try {
            statement = connection.createStatement();
            System.out.println(statementString);
            statement.execute(statementString);
            statement.close();
        } catch (SQLException e) {
        }
    }

    private void executeSQL(String statementString) throws SQLException {
        statement = connection.createStatement();
        System.out.println(statementString);
        statement.execute(statementString);
        statement.close();
    }

    private boolean testExistence(String tableName) {
        try {
            String statementString = "Select COUNT(1) from twineball." + tableName;
            executeSQL(statementString);
        } catch (SQLException e) {
            return false;
        }
        return true;
    }

    /**
     * @throws ClassNotFoundException
     */
    public void loadDriver(String driverName) throws ClassNotFoundException {
        Class.forName(driverName);
    }

}