/*
 * Licensed Materials - Property of IBM
 * 5724-G93
 * (C) Copyright IBM Corporation 2003. All Rights Reserved.
 * US Government Users Restricted Rights- Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 */

/* DISCLAIMER:
 * The following [enclosed] code is sample code created by IBM
 * Corporation. This sample code is not part of any standard IBM product
 * and is provided to you solely for the purpose of assisting you in the
 * development of your applications. The code is provided 'AS IS',
 * without warranty of any kind IBM shall not be liable for any damages
 * arising out of your use of sample code, even if they have been
 * advised of the possibility of such damages.
 */
 package com.ibm.wbia.TwineBall.Server;

import java.util.*;
import java.sql.*;

/**
 * @author Jim Haungs
 */
public class DatabaseTable
{
	private String tableName;
	private Vector columnNames;
	private Hashtable columns;

	public DatabaseTable(String tableName, DatabaseMetaData metadata, String schemaName) throws SQLException {
		this.tableName = tableName.toUpperCase();
		initializeMetadata(metadata, schemaName);
	}

	private void initializeMetadata(DatabaseMetaData metadata, String schemaName) throws SQLException {
		columnNames = new Vector();
		columns = new Hashtable();
		ResultSet rs = metadata.getColumns(null, schemaName, tableName, null);

		while (rs.next()) {
			DatabaseColumn column;

			String name = rs.getString("COLUMN_NAME").toUpperCase();
			String type = rs.getString("TYPE_NAME").toUpperCase();
			int size = rs.getInt("COLUMN_SIZE");

			if (type.equals("INTEGER"))
				column = new DatabaseNumberColumn(this, name, size);
			else
				column = new DatabaseStringColumn(this, name, size);

			columnNames.addElement(name);
			columns.put(name, column);
			}

		rs.close();
		}

	public Vector columnNames()
	{
		return columnNames;
		}

	public DatabaseColumn columnAt (int columnIndex)
	{
		String columnName = (String) columnNames.get(columnIndex);
		return (DatabaseColumn) columns.get(columnName);
		}

	public DatabaseColumn columnNamed (String columnName)
	throws TwineBallException
	{
		DatabaseColumn column = (DatabaseColumn) columns.get(columnName.toUpperCase());
		if (column == null)
			throw new TwineBallException ("A column named \"" + columnName + "\" does not exist in table \"" + tableName + "\"");

		return column;
		}

	public String getColumnType(String columnName)
	throws TwineBallException
	{
		return columnNamed(columnName).getType();
		}

	public int getColumnWidth(String columnName)
	throws TwineBallException
	{
		return columnNamed(columnName).getWidth();
		}

	public Vector getRow (ResultSet rs)
	throws SQLException
	{
	Vector results = new Vector (columnNames.size());

		for (Iterator keys = columns.keySet().iterator(); keys.hasNext();) {
			DatabaseColumn column = (DatabaseColumn) columns.get (keys.next());
			String value = column.getValue (rs);
			String pair = column.getName() + "=" + column.getValue(rs);
			results.addElement (pair);
			}

		return results;
		}

	public String valueList (Vector object)
	throws TwineBallException
	{
	StringBuffer valueList = new StringBuffer();
	Vector values = removePrimaryKey (object);

		for (Iterator iter = values.iterator(); iter.hasNext();) {
			String pair = (String) iter.next();
			DatabaseColumn column = columnNamed (extractName(pair));
			valueList.append (column.sqlValue (extractValue (pair)));
			if (iter.hasNext())
				valueList.append (", ");
			}

		return valueList.toString();
		}

	public String nameList (Vector object)
	{
	StringBuffer nameList = new StringBuffer();
	Vector values = removePrimaryKey (object);

		for (Iterator iter = values.iterator(); iter.hasNext();) {
			String pair = (String) iter.next();
			nameList.append (extractName (pair));
			if (iter.hasNext())
				nameList.append (", ");
			}

		return nameList.toString();
		}

	public String setList (Vector object)
	throws TwineBallException
	{
		return pairList (removePrimaryKey (object), ", ");
		}

	public String searchList (Vector object)
	throws TwineBallException
	{
		return pairList (removeNulls (object), " AND ");
		}

	private String pairList (Vector pairs, String separator)
	throws TwineBallException
	{
	StringBuffer list = new StringBuffer();

		for (Iterator iter = pairs.iterator(); iter.hasNext();) {
			String pair = (String) iter.next();
			String columnName = extractName (pair);
			DatabaseColumn column = columnNamed (columnName);

			list.append (columnName);
			list.append("=");
			list.append(column.sqlValue (extractValue (pair)));

			if (iter.hasNext())
				list.append (separator);
			}

		return list.toString();
		}

	private Vector removePrimaryKey(Vector object)
	{
		Vector values = (Vector) object.clone();

		for (int i = 0; i < values.size(); i++) {
			String pair = (String) values.elementAt(i);
			String columnName = extractName(pair);

			if (columnName.equalsIgnoreCase("primaryKey")) {
				values.removeElementAt(i);
				break;
				}
			}

		return values;
		}

	private Vector removeNulls(Vector values)
	{
		Vector result = new Vector (values.size());

		for (Iterator iter = values.iterator(); iter.hasNext();) {
			String pair = (String) iter.next();

			String value = extractValue (pair);
			if (value.equalsIgnoreCase ("null"))
				continue;

			result.addElement (pair);
			}

		return result;
		}

	private String extractName (String nameValuePair)
	{
		return nameValuePair.substring(0, nameValuePair.indexOf('='));
		}

	private String extractValue (String nameValuePair)
	{
		return nameValuePair.substring(nameValuePair.indexOf('=') + 1);
		}
	}
