/*
 * Licensed Materials - Property of IBM 5724-G93 (C) Copyright IBM Corporation
 * 2003. All Rights Reserved. US Government Users Restricted Rights- Use,
 * duplication or disclosure restricted by GSA ADP Schedule Contract with IBM
 * Corp.
 */
/*
 * DISCLAIMER: The following [enclosed] code is sample code created by IBM
 * Corporation. This sample code is not part of any standard IBM product and is
 * provided to you solely for the purpose of assisting you in the development of
 * your applications. The code is provided 'AS IS', without warranty of any kind
 * IBM shall not be liable for any damages arising out of your use of sample
 * code, even if they have been advised of the possibility of such damages.
 */
package com.ibm.wbia.TwineBall.Server;

import java.rmi.*;
import java.rmi.server.*;
import java.sql.*;
import java.util.*;

import javax.sql.*;
import javax.transaction.xa.*;

/**
 * @author tnelson/jhaungs
 */
public class TwineBallConnection extends Object implements TwineBallInterface,
		Remote {
	private Connection connection;

	private XAResource xaResource;

	private boolean tracing = false;

	private Properties properties;

	private int clientNumber;

	private String currentUserID;

	private Hashtable databaseTables;

	private String schemaName;
	
	XAConnection xaConnection;

	public TwineBallConnection(Properties properties) throws RemoteException {
		this(1, properties);
	}

	public TwineBallConnection(int clientNumber, Properties properties)
			throws RemoteException {
		this.properties = properties;
		this.clientNumber = clientNumber;
		try {
			tracing = getRequiredProperty("enable.tracing").equalsIgnoreCase(
					"true");
		} catch (TwineBallException ex) {
			tracing = true;
		}
	}


	public void connect(String login, String password, String schemaName) throws TwineBallException {
		try {
			if (schemaName == null) {
				schemaName = "twineball";
			}
			String url = getRequiredProperty("database.url");
			traceWrite("Connecting to database: " + url + " as " + login);
			currentUserID = login;
			DatabaseConnection databaseConnection = new DatabaseConnection();
			connection = databaseConnection.getJDBCConnection(login, password, schemaName, url, properties, tracing);
			xaConnection = databaseConnection.getXaConnection();
			xaResource = databaseConnection.getXaResource();
			databaseTables = initializeMetadata();
			setSchemaName(schemaName);
			traceWrite("User " + login + " connected to: " + url);
			currentUserID = login;

		} catch (Exception e) {
			e.printStackTrace();
			throw new TwineBallException(e.getMessage());
		}
	}

	private Hashtable initializeMetadata() throws TwineBallException {
		Hashtable tables = new Hashtable();
		String[] types = { "TABLE" };
		try {
			DatabaseMetaData metadata = connection.getMetaData();
			ResultSet rs = metadata.getTables(null, schemaName, null, types);
			while (rs.next()) {
				String tableName = rs.getString("TABLE_NAME").toUpperCase();
				DatabaseTable table = new DatabaseTable(tableName, metadata,
						schemaName);
				tables.put(tableName, table);
			}
			rs.close();
		} catch (SQLException e) {
			e.printStackTrace();
			throw new TwineBallException(e.getMessage());
		}
		return tables;
	}


	public void disconnect() throws TwineBallException {
		if (!connected())
			return;
		try {
			traceWrite("Closing JDBC connection\n");
			connection.commit();
			connection.close();
		} catch (SQLException e) {
			e.printStackTrace();
			throw new TwineBallException(e.toString());
		}
	}

	public boolean connected() {
		return connection != null;
	}

	public void commit() throws TwineBallException {
		if (!connected())
			return;
		try {
			traceWrite("Commit");
			connection.commit();
		} catch (SQLException e) {
			e.printStackTrace();
			throw new TwineBallException(e.getMessage());
		}
	}

	public void rollback() throws TwineBallException {
		if (!connected())
			return;
		try {
			traceWrite("Rollback");
			connection.rollback();
		} catch (SQLException e) {
			e.printStackTrace();
			throw new TwineBallException(e.getMessage());
		}
	}

	public String insertObject(String entityName, Vector values)
			throws TwineBallException {
		DatabaseTable table = tableNamed(entityName);
		executeSQL("INSERT INTO " + qualifyTable(entityName) + " ("
				+ table.nameList(values) + ")" + " VALUES ("
				+ table.valueList(values) + ")");
		int generatedKey = getIntegerValue("select IDENTITY_VAL_LOCAL() from "
				+ qualifyTable(entityName));
		traceWrite("Generated primary key " + generatedKey + " for "
				+ entityName);
		return String.valueOf(generatedKey);
	}

	public void updateObject(String entityName, String Key, Vector values)
			throws TwineBallException {
		DatabaseTable table = tableNamed(entityName);
		if (values.isEmpty())
			return;
		String updateList = table.setList(values);
		executeSQL("UPDATE " + qualifyTable(entityName) + " SET " + updateList
				+ " WHERE primarykey=" + Key);
	}

	public void deleteObject(String entityName, String primaryKey)
			throws TwineBallException {
		Vector childEntities = getChildNames(entityName);
		for (Iterator entity = childEntities.iterator(); entity.hasNext();) {
			String childEntity = (String) entity.next();
			Vector childKeys = findChildKeys(entityName, primaryKey,
					childEntity);
			for (Iterator child = childKeys.iterator(); child.hasNext();) {
				String childKey = (String) child.next();
				deleteObject(childEntity, childKey);
			}
		}
		executeSQL("DELETE FROM " + qualifyTable(entityName)
				+ " WHERE primarykey = " + primaryKey);
	}

	public Vector getObject(String entityName, String key)
			throws TwineBallException {
		if (key == null || key.equalsIgnoreCase("null"))
			throw new TwineBallException("Attempt to retrieve from '"
					+ entityName + "' using a NULL primary key.");
		DatabaseTable table = tableNamed(entityName);
		String SQLStatement = "SELECT * from " + qualifyTable(entityName)
				+ " where primarykey = " + key;
		try {
			Statement stmt = connection.createStatement();
			traceWrite(SQLStatement);
			ResultSet rs = stmt.executeQuery(SQLStatement);
			Vector results = new Vector();
			if (rs.next())
				results = table.getRow(rs);
			rs.close();
			stmt.close();
			return results;
		} catch (SQLException e) {
			e.printStackTrace();
			throw new TwineBallException(e.getMessage());
		}
	}

	public Vector findObjectKeysByContent(String entityName, Vector values)
			throws TwineBallException {
		DatabaseTable table = tableNamed(entityName);
		String sql = "SELECT primarykey from " + qualifyTable(entityName);
		String whereClause = table.searchList(values);
		if (whereClause.length() > 0)
			sql = sql + " where " + whereClause;
		return findKeys(sql);
	}

	public Vector findObjectKeys(String entityName) throws TwineBallException {
		return findKeys("SELECT primarykey from " + qualifyTable(entityName));
	}

	public Vector findChildKeys(String parentEntity, String parentKey,
			String childEntity) throws TwineBallException {
		if (parentKey == null || parentKey.equalsIgnoreCase("null"))
			return new Vector();
		;
		return findKeys("SELECT primarykey from " + qualifyTable(childEntity)
				+ " where parentkey=" + parentKey + " AND parenttable='"
				+ parentEntity.toUpperCase() + "'");
	}

	private Vector findKeys(String SQLStatement) throws TwineBallException {
		return getIntegers(SQLStatement, "primarykey");
	}

	public Vector getRootObjects() throws TwineBallException {
		return getStrings("SELECT distinct parent from "
				+ qualifyTable("parentchild ")
				+ " where parent not in (SELECT child from " + schemaName
				+ ".parentchild)", "parent");
	}

	public int getObjectCount(String entityName) throws TwineBallException {
		return getIntegerValue("select count(*) from "
				+ qualifyTable(entityName));
	}

	private Vector getStrings(String sql, String columnName)
			throws TwineBallException {
		Vector results = new Vector();
		try {
			Statement stmt = connection.createStatement();
			traceWrite(sql);
			ResultSet rs = stmt.executeQuery(sql);
			while (rs.next()) {
				results.addElement(rs.getString(columnName).toUpperCase());
			}
			rs.close();
			stmt.close();
		} catch (SQLException e) {
			e.printStackTrace();
			throw new TwineBallException(e.getMessage());
		}
		return results;
	}

	private Vector getIntegers(String SQLStatement, String columnName)
			throws TwineBallException {
		Vector results = new Vector();
		try {
			Statement stmt = connection.createStatement();
			traceWrite(SQLStatement);
			ResultSet rs = stmt.executeQuery(SQLStatement);
			while (rs.next())
				results.addElement(String.valueOf(rs.getInt(columnName)));
			rs.close();
			stmt.close();
		} catch (SQLException e) {
			e.printStackTrace();
			throw new TwineBallException(e.getMessage());
		}
		return results;
	}

	private int getIntegerValue(String sql) throws TwineBallException {
		int value = 0;
		try {
			Statement stmt = connection.createStatement();
			traceWrite(sql);
			ResultSet rs = stmt.executeQuery(sql);
			if (rs.next())
				value = rs.getInt(1);
			rs.close();
			stmt.close();
		} catch (SQLException e) {
			e.printStackTrace();
			throw new TwineBallException(e.getMessage());
		}
		return value;
	}

	public Vector getParentNames(String EntityName) throws TwineBallException {
		return getStrings("SELECT parent from " + schemaName
				+ ".parentchild where UPPER(child) = UPPER('" + EntityName
				+ "')", "parent");
	}

	public Vector getChildNames(String EntityName) throws TwineBallException {
		return getStrings("SELECT child from " + schemaName
				+ ".parentchild where UPPER(parent) = UPPER('" + EntityName
				+ "')", "child");
	}

	private DatabaseTable tableNamed(String tableName) {
		System.out.println("table name:" + tableName);
		return (DatabaseTable) databaseTables.get(tableName.toUpperCase());
	}

	private DatabaseColumn columnNamed(String tableName, String columnName)
			throws TwineBallException {
		DatabaseTable table = tableNamed(tableName);
		return table.columnNamed(columnName);
	}

	public Vector getEntityNames() throws TwineBallException {
		return new Vector(databaseTables.keySet());
	}

	public Vector getColumnNames(String tableName) throws TwineBallException {
		traceWrite("Retrieving column names for " + tableName);
		DatabaseTable table = tableNamed(tableName);
		if (table == null)
			return new Vector();
		else
			return table.columnNames();
	}

	public String getColumnType(String entityName, String attributeName)
			throws TwineBallException {
		traceWrite("Retrieving column type for " + entityName + "."
				+ attributeName);
		DatabaseColumn column = columnNamed(entityName, attributeName);
		return column.getType();
	}

	public int getColumnWidth(String entityName, String attributeName)
			throws TwineBallException {
		traceWrite("Retrieving column width for " + entityName + "."
				+ attributeName);
		DatabaseColumn column = columnNamed(entityName, attributeName);
		return column.getWidth();
	}

	private void executeSQL(String SQLStatement) throws TwineBallException {
		try {
			Statement stmt = connection.createStatement();
			traceWrite(SQLStatement);
			stmt.executeUpdate(SQLStatement);
			stmt.close();
		} catch (SQLException e) {
			e.printStackTrace();
			throw new TwineBallException(e.getMessage());
		}
	}

	public int getClientID() throws RemoteException {
		return clientNumber;
	}

	public String getUserID() throws RemoteException {
		return currentUserID;
	}

	public void enableTracing(boolean tracing) throws RemoteException {
		this.tracing = tracing;
	}

	private String getProperty(String propName) {
		return properties.getProperty(propName);
	}

	private String getRequiredProperty(String propName)
			throws TwineBallException {
		String property = getProperty(propName);
		if (property == null)
			throw new TwineBallException("Required property value '" + propName
					+ "' + missing from property file.");
		return property;
	}

	private void traceWrite(String traceString) {
		if (tracing)
			System.out.println("Server #" + clientNumber + ": " + traceString);
	}

	public void setSchemaName(String schemaName) throws TwineBallException {
		this.schemaName = schemaName.toUpperCase();
		try {
			Statement statement = connection.createStatement();
			statement.execute("SET SCHEMA " + schemaName);
		} catch (Exception e) {
			throw new TwineBallException(e.getMessage());
		}
		traceWrite("changing default schema to " + schemaName);
	}

	private String qualifyTable(String entityName) {
		String newEntityName = schemaName + "." + entityName;
		return newEntityName;
	}

	/**
	 * @return Returns the xaResource.
	 */
	public XAResource getXAResource() {
		return xaResource;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.transaction.xa.XAResource#commit(javax.transaction.xa.TwineBallXid,
	 *      boolean)
	 */
	public void commit(TwineBallXid xid, boolean onePhase) throws XAException {
		traceWrite("XA Commit" + xid.globalTransactionId);
		xaResource.commit(xid, onePhase);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.transaction.xa.XAResource#end(javax.transaction.xa.TwineBallXid,
	 *      int)
	 */
	public void end(TwineBallXid xid, int flags) throws XAException {
		traceWrite("XA end" + xid.globalTransactionId);
		xaResource.end(xid, flags);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.transaction.xa.XAResource#forget(javax.transaction.xa.TwineBallXid)
	 */
	public void forget(TwineBallXid xid) throws XAException {
		traceWrite("XA Forget" + xid.globalTransactionId);
		xaResource.forget(xid);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.transaction.xa.XAResource#getTransactionTimeout()
	 */
	public int getTransactionTimeout() throws XAException {
		traceWrite("XA getTransactionTimeout");
		return xaResource.getTransactionTimeout();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.transaction.xa.XAResource#isSameRM(javax.transaction.xa.XAResource)
	 */
	public boolean isSameRM(XAResource xares) throws XAException {
		traceWrite("XA isSameRM");
		return xaResource.isSameRM(xares);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.transaction.xa.XAResource#prepare(javax.transaction.xa.TwineBallXid)
	 */
	public int prepare(TwineBallXid xid) throws XAException {
		traceWrite("XA Prepare: " + xid.globalTransactionId);
		return xaResource.prepare(xid);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.transaction.xa.XAResource#recover(int)
	 */
	public TwineBallXid[] recover(int flag) throws XAException {
		traceWrite("XA Recover: flag:" + flag);
		Xid[] xids = xaResource.recover(flag);
		TwineBallXid[] TBXids = new TwineBallXid[xids.length];
		for (int i = 0; i < xids.length; i++) {
			TBXids[i] = new TwineBallXid(xids[i]);
		}
		return TBXids;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.transaction.xa.XAResource#rollback(javax.transaction.xa.TwineBallXid)
	 */
	public void rollback(TwineBallXid xid) throws XAException {
		traceWrite("XA Rollback" + xid.globalTransactionId);
		xaResource.rollback(xid);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.transaction.xa.XAResource#setTransactionTimeout(int)
	 */
	public boolean setTransactionTimeout(int seconds) throws XAException {
		traceWrite("XA set Tranaction timeout: " + seconds + " seconds");
		return xaResource.setTransactionTimeout(seconds);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.transaction.xa.XAResource#start(javax.transaction.xa.TwineBallXid,
	 *      int)
	 */
	public void start(TwineBallXid xid, int flags) throws XAException {
		traceWrite("XA start: " + xid.globalTransactionId + " flags:" + flags);
		try {
			connection.setAutoCommit(true);
			xaResource.start(xid, flags);

			//connection = xaConnection.getConnection();
			//setSchemaName(schemaName);
			//databaseTables = initializeMetadata();
			//connection.setAutoCommit(false);
		} catch (XAException e) {
			traceWrite("XA Exception: " + e.getMessage());
			traceWrite("Error code:" + e.errorCode);
			if (e.errorCode == XAException.XAER_PROTO) {
				traceWrite("Method invoked in improper context");
			}
			e.printStackTrace();
			throw e;
		} catch (Exception e) {
			traceWrite("SQL Exception: " + e.getMessage());

			e.printStackTrace();
			throw new XAException(e.getMessage());
		}
	}
}