/*
 * Licensed Materials - Property of IBM
 * 5724-G93
 * (C) Copyright IBM Corporation 2003. All Rights Reserved.
 * US Government Users Restricted Rights- Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 */

/* DISCLAIMER:
 * The following [enclosed] code is sample code created by IBM
 * Corporation. This sample code is not part of any standard IBM product
 * and is provided to you solely for the purpose of assisting you in the
 * development of your applications. The code is provided 'AS IS',
 * without warranty of any kind IBM shall not be liable for any damages
 * arising out of your use of sample code, even if they have been
 * advised of the possibility of such damages.
 */

package com.ibm.wbia.TwineBall.Server;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.Enumeration;
import java.util.MissingResourceException;
import java.util.Properties;
import java.util.ResourceBundle;

public class TwineBallProperties {
	
	public  Properties getProperties(String propertyFile){
		Properties properties;
		String systemPropertyFileName = System.getProperty("PropertyFile");
		if (propertyFile != null){ //property file was given as an argument. read it.
			properties = getPropertiesFromFile(propertyFile);
		}else if (systemPropertyFileName != null){
			properties = getPropertiesFromFile(systemPropertyFileName);
		}else{
			properties = getPropertiesFromClasspath();
		}
		return properties;
		
	}
	/**
	 * @return
	 */
	private Properties getPropertiesFromClasspath() {
		Properties properties;
		try{
			properties = new Properties();
			ResourceBundle bundle = ResourceBundle.getBundle("twineball"); 
		   	Enumeration propertyIterator = bundle.getKeys();
			while (propertyIterator.hasMoreElements()){
			    String propertyName = (String)propertyIterator.nextElement();
			    String value = bundle.getString(propertyName);
			    properties.put(propertyName,value);
			}
			
		}catch(MissingResourceException e){
			System.err.println("No property file name passed in, could not find TwineBall.properties: Using defaults.");
			properties = getDefaultProperties();
		}
		return properties;
	}
	public Properties getProperties(){
		return getProperties(null);
	}
	
	
	private  Properties getDefaultProperties(){
		Properties properties = new Properties();
		properties.setProperty("database.driver","org.apache.derby.jdbc.EmbeddedDriver");
		properties.setProperty("enable.tracing","true");
		properties.setProperty("database.url","jdbc:derby:twine;create=true");
        return properties;
	}
	private Properties getPropertiesFromFile(String propertyFile) {
		Properties properties = new Properties();
		try {
			properties.load(new FileInputStream(new File(propertyFile)));
		} catch (IOException ex) {
			System.err.println("Unable to open properties file " + propertyFile + ": " + ex);
		}
		return properties;
	}

}
