/*
 * Licensed Materials - Property of IBM
 * 5724-G93
 * (C) Copyright IBM Corporation 2003. All Rights Reserved.
 * US Government Users Restricted Rights- Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 */

/* DISCLAIMER:
 * The following [enclosed] code is sample code created by IBM
 * Corporation. This sample code is not part of any standard IBM product
 * and is provided to you solely for the purpose of assisting you in the
 * development of your applications. The code is provided 'AS IS',
 * without warranty of any kind IBM shall not be liable for any damages
 * arising out of your use of sample code, even if they have been
 * advised of the possibility of such damages.
 */

package com.ibm.wbia.TwineBall.Server;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.net.InetAddress;
import java.net.MalformedURLException;
import java.rmi.Naming;
import java.rmi.RMISecurityManager;
import java.rmi.RemoteException;
import java.rmi.registry.LocateRegistry;
import java.rmi.registry.Registry;
import java.rmi.server.UnicastRemoteObject;
import java.sql.SQLException;
import java.util.Enumeration;
import java.util.MissingResourceException;
import java.util.Properties;
import java.util.ResourceBundle;
import java.util.StringTokenizer;

public class TwineBallServer extends UnicastRemoteObject implements TwineBallServerInterface {
	private static Properties properties;
	private static int clientCounter;
	private static Registry registry;
	private final static String serverName = "TwineBallServer";
	private final static int defaultPort = 2112;
	private static boolean databaseChecked = false;

	public TwineBallServer(Properties properties) throws RemoteException {
		super();
		TwineBallServer.properties = properties;
	}

	public TwineBallInterface connect(String login, String password, String schemaName) throws TwineBallException, RemoteException {
		clientCounter++;

		TwineBallInterface server = new TwineBallConnection(clientCounter, properties);
		
		server.connect(login, password, schemaName);
		UnicastRemoteObject.exportObject(server);
		System.out.println("User " + login + " connected to server #" + clientCounter);

		return server;
	}
	
	


	private static String hostName() {
		String hostName;

		try {
			hostName = InetAddress.getLocalHost().getHostName();
		} catch (Exception ex) {
			hostName = "localhost";
		}

		return hostName;
	}

	public static String defaultURL() {
		return "//" + hostName() + ":" + defaultPort + "/" + serverName;
	}

	private static int portNumber(String URL) {
		StringTokenizer stream = new StringTokenizer(URL, ":/", true);

		while (stream.hasMoreTokens()) {
			String token = stream.nextToken();
			if (token.equals(":")) {
				if (stream.hasMoreTokens()) {
					token = stream.nextToken();
					return Integer.decode(token).intValue();
				}
			}
		}

		return defaultPort;
	}




	private static void createServer() throws RemoteException, MalformedURLException {
		String url = getProperty("server.name");
		if (url == null)
			url = defaultURL();

		System.out.println("Creating server on URL " + url);

		int port = portNumber(url);
		
		if (System.getSecurityManager() == null)
			System.setSecurityManager(new RMISecurityManager());
		System.out.println("Creating RMI registry on port " + port);
		registry = LocateRegistry.createRegistry(port);

		System.out.println("Creating Server " + url);
		TwineBallServer factory = new TwineBallServer(properties);

		System.out.println("Binding to Server " + url);
		Naming.rebind(url, factory);

		System.out.println("Server " + url + " has been registered.");
	}

	private static void loadDatabaseDriver() throws TwineBallException, ClassNotFoundException {
		String className = getRequiredProperty("database.driver");
		System.out.println("Loading JDBC driver: " + className);
		Class.forName(className);
		System.out.println("Successfully loaded JDBC driver: " + className);
	}

	private static String getProperty(String propName) {
		return properties.getProperty(propName);
	}

	private static String getRequiredProperty(String propName) throws TwineBallException {
		String property = getProperty(propName);

		if (property == null)
			throw new TwineBallException("Required property value '" + propName + "' + missing from property file.");

		return property;
	}

	public static void main(String[] args) {

		try {
			TwineBallProperties twineBallProperties = new TwineBallProperties();
			if (args.length > 0){
				properties = twineBallProperties.getProperties(args[0]);
			} else{
				properties = twineBallProperties.getProperties();
			}
			loadDatabaseDriver();
	        DatabaseCreation database= new DatabaseCreation();
	        database.createDatabase(properties);
	        databaseChecked=true;
	        clientCounter=0;
			createServer();
		} catch (Exception ex) {
		    ex.printStackTrace();
			System.out.println(ex.getMessage());
			System.exit(1);
		}
	}
}
