/*
 * Licensed Materials - Property of IBM pidnumber (C) Copyright IBM Corporation
 * 2005. All Rights Reserved. US Government Users Restricted Rights- Use,
 * duplication or disclosure restricted by GSA ADP Schedule Contract with IBM
 * Corp.
 */
package com.ibm.j2ca.sample.twineball.inbound;

import java.util.Iterator;
import java.util.Vector;
import java.util.logging.Logger;

import javax.resource.spi.BootstrapContext;
import javax.resource.spi.endpoint.MessageEndpointFactory;
import javax.sql.XADataSource;

import junit.framework.TestCase;

import com.ibm.j2ca.extension.eventmanagement.EventDistributionResourceDBImpl;
import com.ibm.j2ca.extension.logging.LogLevel;
import com.ibm.j2ca.sample.twineball.TwineBallResourceAdapter;
import com.ibm.j2ca.sample.twineball.mocks.MockBootstrapContext;
import com.ibm.j2ca.sample.twineball.mocks.MockConcreteListener;
import com.ibm.j2ca.sample.twineball.mocks.MockMessageEndpointFactory;
import com.ibm.j2ca.sample.twineball.mocks.TwineBallActivationSpecFactory;
import com.ibm.wbia.TwineBall.Server.TwineBallConnectionFactory;
import com.ibm.wbia.TwineBall.Server.TwineBallInterface;

public class TestPolling extends TestCase {
    TwineBallResourceAdapter resourceAdapter;

    private EventDistributionResourceDBImpl table;

    XADataSource xaDS;

    TwineBallInterface eisConnection;

    protected void setUp() throws Exception {
        super.setUp();
        TwineBallConnectionFactory factory = new TwineBallConnectionFactory();
        eisConnection = factory.getConnection("twineball", "sample42", "twineball", "Cloudscape");
        Vector currentEvents = eisConnection.findObjectKeys("WBIA_EVENTS");
        Iterator eventIterator = currentEvents.iterator();
        while (eventIterator.hasNext()) {
            String currentEvent = (String) eventIterator.next();
            eisConnection.deleteObject("WBIA_EVENTS", currentEvent);
            eisConnection.commit();
        }
        resourceAdapter = new TwineBallResourceAdapter();
        resourceAdapter.setLogFilename("LOG");
        resourceAdapter.setTraceFilename("TRACE");
        resourceAdapter.setAdapterID("TwineBall");

        BootstrapContext bootstrapContext = new MockBootstrapContext();
        Logger.getLogger("com.ibm.j2ca.sample.twineball.log.TwineBall").setLevel(LogLevel.ALL);
        Logger.getLogger("com.ibm.j2ca.sample.twineball.cei.TwineBall").setLevel(LogLevel.OFF);
        resourceAdapter.start(bootstrapContext);
    }

    /*
     * @see TestCase#tearDown()
     */
    protected void tearDown() throws Exception {
        super.tearDown();
    }

    public void testPollingNonTransactional() throws Exception {
        Vector record1 = new Vector();
        record1.add("FirstName=Travis");
        record1.add("LastName=Nelson");
        eisConnection.insertObject("Customer", record1);
        eisConnection.commit();
        int count = eisConnection.getObjectCount("WBIA_EVENTS");
        assertTrue("count is :" + count, count == 1);
        MessageEndpointFactory endpointFactory = new MockMessageEndpointFactory();
        TwineBallActivationSpec activationSpec = TwineBallActivationSpecFactory.createActivationSpec();
        activationSpec.eventTableName = "WBIA_EVENTS";
        activationSpec.schema = "twineball";
        activationSpec.setResourceAdapter(resourceAdapter);
        activationSpec.setBONamespace("http://www.ibm.com/xmlns/prod/websphere/j2ca/twineball/customer");
        resourceAdapter.endpointActivation(endpointFactory, activationSpec);
        Object endpoint = endpointFactory.createEndpoint(null);
        synchronized (endpoint) {
            endpoint.wait(500000);
            resourceAdapter.endpointDeactivation(endpointFactory, activationSpec);
            resourceAdapter.stop();
        }
        assertTrue("messages recieved:" + MockConcreteListener.recievedMessages, MockConcreteListener.recievedMessages == 1);
    }

    public void testPollingTransactional() throws Exception {
        Vector record1 = new Vector();
        record1.add("FirstName=Travis");
        record1.add("LastName=Nelson");
        eisConnection.insertObject("Customer", record1);
        eisConnection.commit();
        int count = eisConnection.getObjectCount("WBIA_EVENTS");
        assertTrue("count is :" + count, count == 1);
        MockMessageEndpointFactory endpointFactory = new MockMessageEndpointFactory(true);
        TwineBallActivationSpec activationSpec = TwineBallActivationSpecFactory.createActivationSpec();
        activationSpec.eventTableName = "WBIA_EVENTS";
        activationSpec.schema = "twineball";
        activationSpec.setResourceAdapter(resourceAdapter);
        activationSpec.setBONamespace("http://www.ibm.com/xmlns/prod/websphere/j2ca/twineball/customer");
        resourceAdapter.endpointActivation(endpointFactory, activationSpec);
        while(endpointFactory.getListener() == null){
            Thread.sleep(10);
        }
        MockConcreteListener endpoint = endpointFactory.getListener();
        synchronized (endpoint) {
            endpoint.wait(500000);
            resourceAdapter.endpointDeactivation(endpointFactory, activationSpec);
            resourceAdapter.stop();
        }
        assertTrue("messages recieved:" + MockConcreteListener.recievedMessages, MockConcreteListener.recievedMessages == 1);
    }

    public void testPolling2Phase() throws Exception {
        Vector record1 = new Vector();
        record1.add("FirstName=Travis");
        record1.add("LastName=Nelson");
        eisConnection.insertObject("Customer", record1);
        eisConnection.commit();
        int count = eisConnection.getObjectCount("WBIA_EVENTS");
        assertTrue("count is :" + count, count == 1);
        MockMessageEndpointFactory endpointFactory = new MockMessageEndpointFactory(true);
        endpointFactory.setTwoPhase(true);
        TwineBallActivationSpec activationSpec = TwineBallActivationSpecFactory.createActivationSpec();
        activationSpec.eventTableName = "WBIA_EVENTS";
        activationSpec.schema = "twineball";
        activationSpec.setResourceAdapter(resourceAdapter);
        activationSpec.setBONamespace("http://www.ibm.com/xmlns/prod/websphere/j2ca/twineball/customer");
        resourceAdapter.endpointActivation(endpointFactory, activationSpec);
        while(endpointFactory.getListener() == null){
            Thread.sleep(10);
        }
        MockConcreteListener endpoint = endpointFactory.getListener();
        synchronized (endpoint) {
            endpoint.wait(500000);
            resourceAdapter.endpointDeactivation(endpointFactory, activationSpec);
            resourceAdapter.stop();
        }
        assertTrue("messages recieved:" + MockConcreteListener.recievedMessages, MockConcreteListener.recievedMessages == 1);
    }
}