/*
 * Licensed Materials - Property of IBM
 * pidnumber
 * (C) Copyright IBM Corporation 2005. All Rights Reserved.
 * US Government Users Restricted Rights- Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 */
package com.ibm.j2ca.sample.twineball.inbound;

import java.util.List;
import java.util.logging.Logger;
import javax.resource.spi.BootstrapContext;
import junit.framework.TestCase;
import com.ibm.j2ca.base.WBIRecord;
import com.ibm.j2ca.extension.eventmanagement.Event;
import com.ibm.j2ca.extension.logging.LogLevel;
import com.ibm.j2ca.sample.twineball.TwineBallResourceAdapter;
import com.ibm.j2ca.sample.twineball.mocks.MockBootstrapContext;
import com.ibm.j2ca.sample.twineball.outbound.TwineBallConnection;
import com.ibm.j2ca.sample.twineball.outbound.TwineBallConnectionFactory;
import com.ibm.j2ca.sample.twineball.outbound.TwineBallConnectionSpec;
import com.ibm.j2ca.sample.twineball.outbound.TwineBallManagedConnectionFactory;
import commonj.sdo.DataObject;

public class TwineBallEventStoreTest extends TestCase {
	
	public static final String TABLE = "WBIA_EVENTS";
	private TwineBallEventStore eventStore;
	private TwineBallConnection connection;
	/*
	 * @see TestCase#setUp()
	 */
	protected void setUp() throws Exception {
		super.setUp();
		
		TwineBallActivationSpec activationSpec = new TwineBallActivationSpec();
		
		// Get a ConnectionFactory instance:
		TwineBallManagedConnectionFactory managedConnectionFactory = new TwineBallManagedConnectionFactory();
		TwineBallConnectionSpec connectionSpec;
		TwineBallConnectionFactory connectionFactory;
		activationSpec.setUrl("MockConnection");
		activationSpec.setUserName("twineball");
		activationSpec.setPassword("media1a"); 
		activationSpec.setSchema("twineball");
		activationSpec.setEventTableName("WBIA_EVENTS");
		TwineBallResourceAdapter resourceAdapter = new TwineBallResourceAdapter();
		resourceAdapter.setLogFilename("LOG");
		resourceAdapter.setTraceFilename("TRACE");
		resourceAdapter.setAdapterID("TwineBall");
		activationSpec.setBONamespace("http://www.ibm.com/xmlns/prod/websphere/j2ca/twineball/customer");
		BootstrapContext bootstrapContext = new MockBootstrapContext();
		Logger.getLogger("com.ibm.j2ca.sample.twineball.log.TwineBall").setLevel(LogLevel.ALL);
		Logger.getLogger("com.ibm.j2ca.sample.twineball.cei.TwineBall").setLevel(LogLevel.OFF);
		resourceAdapter.start(bootstrapContext);
		eventStore = new TwineBallEventStore(activationSpec,resourceAdapter);
	}


	/*
	 * @see TestCase#tearDown()
	 */
	protected void tearDown() throws Exception {
		super.tearDown();
	}

	/*
	 * Class to test for ArrayList getEvents(int, int)
	 */
	public void testGetEventsintint()throws Exception {
		List events= eventStore.getEvents(2,Event.NEWEVENT, null);
		assertTrue(events.size()==2);
	}


	public void testDeleteEvent() throws Exception {
		Event deletableEvent = new Event();
		deletableEvent.setEventId(""+1);
		eventStore.deleteEvent(deletableEvent);
	}

	public void testGetObjectForEvent()throws Exception {
		TwineBallEvent e = new TwineBallEvent();
		e.setEventType("Customer");
		e.setEventKeys("00001");
		Object retrievedObject = eventStore.getObjectForEvent(e);
		assertTrue(retrievedObject instanceof WBIRecord);
		WBIRecord outRecord = (WBIRecord)retrievedObject;
		DataObject dataObject = outRecord.getDataObject().getDataObject("Customer");
		assertTrue(dataObject.getString("Primarykey").equals("00001"));
		assertTrue(dataObject.getString("Firstname").equalsIgnoreCase("travis"));	
	}
}
