/*
 * IBM Confidential
 * OCO Source Materials
 * 5724L78,5724L81,5724L77,5724L79,5724L80
 * (C) Copyright IBM Corporation 2005
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has been
 * deposited with the U.S. Copyright Office.
 */
package com.ibm.j2ca.sample.twineball.mocks;

import java.lang.reflect.Method;
import java.util.List;

import javax.resource.ResourceException;
import javax.resource.cci.Record;
import javax.resource.spi.endpoint.MessageEndpoint;
import javax.transaction.xa.XAException;
import javax.transaction.xa.XAResource;
import javax.transaction.xa.Xid;

import com.ibm.j2ca.base.WBIRecord;
import com.ibm.j2ca.uthelpers.TestTracer;
import commonj.connector.runtime.InboundListener;


public class MockConcreteListener implements MockListener, MessageEndpoint, InboundListener {
    public static int recievedMessages = 0; // this should be changed; hard to differentiate messages to diff. endpoints; if nothing else, encapsualted as a getter method

    private List eventList;
    private XAResource xaResource;
    private Xid xid = null;
    private boolean twoPhase = false;
       
    
    public MockConcreteListener(List eventList, XAResource xaResource) {
    	this.eventList = eventList;
        this.xaResource = xaResource;
        if(xaResource != null) {
            this.xid = new MockXidImpl();
        }
        recievedMessages=0;
    }
    
    public XAResource getXAResource() {
    	return xaResource;
    }

    /**
     * Listener method from InboundListener interface 
     */
    public void onNotification(Record event) {
        TestTracer.traceMethodEntrance();
        processMessageReceived(event);
        TestTracer.traceMethodExit();
    }
    
    /**
     * Listener method from InboundListener interface 
     */
    public Record onMessage(Record event) {
        TestTracer.traceMethodEntrance();
        processMessageReceived(event);
        TestTracer.traceMethodExit();
        return null;
    }
    
    /**
     * Listener method from MockListener interface 
     */
    public void acceptMessage(Object o) {
        TestTracer.traceMethodEntrance();
        processMessageReceived(o);
        TestTracer.traceMethodExit();
    }
    
    private void processMessageReceived(Object event) {
        if(event instanceof WBIRecord) {
            String name = ((WBIRecord)event).getDataObject().getType().getName();
            TestTracer.trace("Received WBIRecord: " + name);
        } else {
            TestTracer.trace("Received non-WBIRecord event: " + event);
        }

        if (eventList != null) {
            synchronized (eventList) {
                eventList.add(event);
                eventList.notifyAll();
            }
        }
        
        synchronized (MockConcreteListener.class) {
            recievedMessages ++; // deprecated counter
           
        }
        synchronized(this){
        this.notify();
        }
    }
    
    //==== Related to MessageEndpoint Implementation

    public void afterDelivery() throws ResourceException {
        TestTracer.traceMethodEntrance();
        if (xaResource != null) {
            try {
                endTransaction();
                if(twoPhase){
                    xaResource.prepare(xid);
                    xaResource.commit(xid, false);
                }
                else{
                    xaResource.commit(xid,true);
                }
            } catch (XAException e) {
                TestTracer.trace(e.getMessage());
                e.printStackTrace();
                throw new ResourceException(e);
            }
        }
        TestTracer.traceMethodExit();
    }

    public void beforeDelivery(Method arg0) throws NoSuchMethodException, ResourceException {
        TestTracer.traceMethodEntrance();
        if (xaResource != null) {
            try {
                xaResource.start(xid, XAResource.TMNOFLAGS);
            } catch (XAException e) {
                TestTracer.trace(e.getMessage());
                e.printStackTrace();
                System.out.println(xid);
                throw new ResourceException(e);
            }
        }
        TestTracer.traceMethodExit();
    }

    public void release() {
        // no op
    }

    //==== Helper Methods for Subclasses
    
    protected void endTransaction() throws XAException {
        xaResource.end(xid, XAResource.TMSUCCESS);
    }

    protected void rollback() throws XAException {
        xaResource.rollback(xid);
    }
    
    /**
     * @return Returns the twoPhase.
     */
    public boolean isTwoPhase() {
        return twoPhase;
    }
    /**
     * @param twoPhase The twoPhase to set.
     */
    public void setTwoPhase(boolean twoPhase) {
        this.twoPhase = twoPhase;
    }
    /**
     * @return Returns the recievedMessages.
     */
    public static int getRecievedMessages() {
        return recievedMessages;
    }
    /**
     * @param recievedMessages The recievedMessages to set.
     */
    public static void setRecievedMessages(int recievedMessages) {
        MockConcreteListener.recievedMessages = recievedMessages;
    }
}