/*
 * IBM Confidential
 * OCO Source Materials
 * 5724L78,5724L81,5724L77,5724L79,5724L80
 * (C) Copyright IBM Corporation 2005
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has been
 * deposited with the U.S. Copyright Office.
 */
package com.ibm.j2ca.sample.twineball.mocks;

import java.lang.reflect.Method;
import java.util.Collections;
import java.util.LinkedList;
import java.util.List;

import javax.resource.spi.UnavailableException;
import javax.resource.spi.endpoint.MessageEndpoint;
import javax.resource.spi.endpoint.MessageEndpointFactory;
import javax.transaction.xa.XAResource;


public class MockMessageEndpointFactory implements MessageEndpointFactory {
	private boolean transacted;
	private boolean available;
	private List eventList;
	private boolean twoPhase = false;
	
	private MockConcreteListener listener;
	
	public MockMessageEndpointFactory() {
		this(false);
	}

	public MockMessageEndpointFactory(boolean transacted) {
	    this.transacted = transacted;
		this.available = true; 
		this.eventList = new LinkedList();
	}
			
    public MessageEndpoint createEndpoint(XAResource xaResource) throws UnavailableException {
    	if(!available) {
    		throw new UnavailableException("This is a MOCK UnavailableException");
    	}
    	if(listener == null){
    	    listener = new MockConcreteListener(eventList, xaResource);
    	} 
    	listener.setTwoPhase(twoPhase);
    	return listener;
    }
    
    public boolean isDeliveryTransacted(Method method) throws NoSuchMethodException {
        return transacted;
    }
        
    
    //==== HOOKS FOR TESTING
    
    /**
     * Returns the number of new messages received by this endpoint since the
     * last invocation of this method.
     */
    public int getNewMessageCount() {
        return getNewMessages().size();
    }
    
    public List getNewMessages() {
        List tmp;
        synchronized(eventList) {
            tmp = new LinkedList(eventList);
            eventList.clear();
        }
        return tmp;
    }
    
    public void waitForEventDelivery(long timeout) throws InterruptedException {
        long startTime = System.currentTimeMillis();
        long maxTime = startTime + timeout;
        synchronized(eventList) {
            while(eventList.isEmpty() && System.currentTimeMillis() < maxTime) {
                long remainingTime = maxTime - System.currentTimeMillis();
                eventList.wait(remainingTime);
            }
        }
    }
    
    public void setAvailable(boolean b) {
    	this.available = b;
    }
    
    public boolean isAvailable() {
    	return available;
    }
    /**
     * @return Returns the twoPhase.
     */
    public boolean isTwoPhase() {
        return twoPhase;
    }
    /**
     * @param twoPhase The twoPhase to set.
     */
    public void setTwoPhase(boolean twoPhase) {
        this.twoPhase = twoPhase;
    }
    /**
     * @return Returns the listener.
     */
    public MockConcreteListener getListener() {
        return listener;
    }
}
