/*
 * IBM Confidential
 * OCO Source Materials
 * 5724L78,5724L81,5724L77,5724L79,5724L80
 * (C) Copyright IBM Corporation 2005
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has been
 * deposited with the U.S. Copyright Office.
 */
package com.ibm.j2ca.sample.twineball.mocks;

import javax.transaction.xa.Xid;

public class MockXidImpl implements Xid {
    private byte[] branchQualifier;

    private byte[] globalTransactionID;

    private int formatID;

    /**
     * Create new Xid instance with random values.
     */
    public MockXidImpl() {
        // Global Transaction Id will be identical for all the branches under
        // the same distributed transaction. Here it is set to letter 'A'
        byte[] globalId = new byte[1];
        globalId[0] = getRandomByte();
        
        // Transaction Branch Id is unique for each branch under the same
        // distributed transaction. Set the transaction id as the parameter
        // passed.
        byte[] tempBranchId = new byte[1];
        tempBranchId[0] = getRandomByte();
        
        byte[] globalTranId = new byte[64];
        byte[] branchQualifier = new byte[64];
        
        // Copy Global Transaction Id and Branch Id into a 64 byte array
        System.arraycopy(globalId, 0, globalTranId, 0, 1);
        System.arraycopy(tempBranchId, 0, branchQualifier, 0, 1);

        // Create the Transaction Id
        // Transaction Id has 3 components
        globalTranId[32] = getRandomByte();
        globalTranId[31] = getRandomByte();
        
        this.formatID=0x1234;
        this.globalTransactionID=globalTranId;
        this.branchQualifier=branchQualifier;
    }
    
    public MockXidImpl(int formatID, byte[] globalTransactionId, byte[] branchQualifier) {
        this.formatID = formatID;
        this.globalTransactionID = globalTransactionId;
        this.branchQualifier = branchQualifier;
    }

    public int getFormatId() {
        return formatID;
    }

    public byte[] getGlobalTransactionId() {
        return globalTransactionID;
    }

    public byte[] getBranchQualifier() {
        return branchQualifier;
    }

    public String toString() {
        String header = "XID: ";
        for (int i = 0; i < globalTransactionID.length; i++) {
            header += globalTransactionID[i];
        }
        header += "---";
        for (int i = 0; i < branchQualifier.length; i++) {
            header += branchQualifier[i];
        }
        return header;
    }
    
    
    private byte getRandomByte() {
        double random = Math.abs(Math.random());
        int randomInt = (int) Math.round(random * 127);
        return (byte) randomInt;
    }
}