/*
 * Licensed Materials - Property of IBM
 * pidnumber
 * (C) Copyright IBM Corporation 2005. All Rights Reserved.
 * US Government Users Restricted Rights- Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 */
package com.ibm.j2ca.sample.twineball.mocks;
import java.io.IOException;
import java.util.Iterator;
import java.util.List;
import javax.resource.ResourceException;
import com.ibm.j2ca.base.AdapterBOUtil;
import com.ibm.j2ca.base.exceptions.BusinessObjectDefinitionNotFoundException;
import com.ibm.j2ca.sample.twineball.emd.StringCaseChanger;
import commonj.sdo.ChangeSummary;
import commonj.sdo.DataObject;
import commonj.sdo.Property;
import commonj.sdo.Type;

public class SDOBuilder {
	public static final String FIRSTNAME = "Firstname";
    public static final String LASTNAME = "Lastname";
    public static final String CITY = "City";
    public static final String ZIPCODE = "Zipcode";
    public static final String SCHEMA = "http://www.ibm.com/xmlns/prod/websphere/j2ca/twineball/customer";
	//private static final String SCHEMA = "http://www.ibm.com/xmlns/prod/websphere/j2ca/twineball/customer";
	public static final String GRAPH_NAME = "CustomerBG";
    public static final String KEY_FIELD = "Primarykey";
    public static final String PRIMARY_DATA_OBJECT = "Customer";
	public static final String cardNChild = "Address";
	
	public SDOBuilder() throws Exception{
	}
	
	
	
	private DataObject createCustomerSingleLevel() throws Exception{	
		String key="00001";
		DataObject dataObj = createCustomer(key);
		return dataObj;
	}
	
	public ChangeSummary beginLogging(DataObject graph){
		ChangeSummary cs = (ChangeSummary) graph.get("changeSummary");
		cs.beginLogging();
		return cs;
	}

	/**
	 * @param key
	 * @return
	 */
	public DataObject createCustomer(String key) throws Exception{
		DataObject dataObj = createGraph();
		addCustomer(key, dataObj);
		return dataObj;
	}

	/**
     * @return
     * @throws IOException
     */
	public DataObject createGraph() throws Exception {
        String objName = GRAPH_NAME;
		DataObject dataObj = AdapterBOUtil.createDataObject(SCHEMA, objName);
        return dataObj;
    }



    /**
     * @param key
     * @param dataObj
     */
    public void addCustomer(String key, DataObject dataObj) {
        DataObject childObj = dataObj.createDataObject(PRIMARY_DATA_OBJECT);
        childObj.setString(FIRSTNAME, "Travis");
        childObj.setString(LASTNAME, "Nelson");
        childObj.setString(KEY_FIELD, key);
    }

    public DataObject addNAddresses(DataObject parentObject, int n) throws Exception {
        DataObject customerObj = parentObject.getDataObject(PRIMARY_DATA_OBJECT);
        Type t = customerObj.getType();
        Property p = t.getProperty(cardNChild);
        for (int i = 0; i < n; i++) {
            DataObject currentObject = customerObj.createDataObject(p);
            currentObject.setString(KEY_FIELD, Integer.toString(i+1));
            currentObject.setString(CITY, "San Mateo");
            currentObject.setString(ZIPCODE, Integer.toString(i));
        }
        return parentObject;
    }


    public String getEntityName(DataObject dataObject) throws ResourceException, BusinessObjectDefinitionNotFoundException {
        DataObject metadata;
        try {
            metadata = AdapterBOUtil.getMetadataForObject(dataObject);
        } catch (Exception e) {
            throw new ResourceException(e);
        }
        return metadata.getString("ObjectName");
    }

    public String getKey(DataObject dataObject) {
        return dataObject.getString(KEY_FIELD);
    }

    public String getChildDataObjectName(DataObject container, String propertyName) throws BusinessObjectDefinitionNotFoundException {
        String containerName = container.getType().getInstanceClass().getName();
        System.out.println(containerName);
        int index = containerName.lastIndexOf(".");
        String childDataObjectName = containerName.substring(0, index + 1) + propertyName.toLowerCase() + "." + StringCaseChanger.toCamelCase(propertyName);
        System.out.println(childDataObjectName);
        return childDataObjectName;
    }

 

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.j2c.comparator.KeyRetriever#setKey(commonj.sdo.DataObject,
     *      java.lang.String)
     */
    public void setKey(DataObject object, String key) {
        object.setString(KEY_FIELD, key);
    }

    public DataObject removeAddress(DataObject parentObject, int index) {
        DataObject customerObj = parentObject.getDataObject(PRIMARY_DATA_OBJECT);
        Type t = customerObj.getType();
        Property p = t.getProperty(cardNChild);
        List list = customerObj.getList(p);
        list.remove(index);
        return parentObject;
    }



    public DataObject getAddress(DataObject parentObject, int addressPosition) {
        DataObject customerObj = getCustomer(parentObject);
        List addressList = customerObj.getList(cardNChild);
        DataObject address = (DataObject) addressList.get(addressPosition);
        return address;
    }

    public DataObject getCustomer(DataObject parentObject) {
        DataObject customerObj = parentObject.getDataObject(PRIMARY_DATA_OBJECT);
        return customerObj;
    }

    public void printChangeSummary(ChangeSummary cs) {
        Iterator dataObjects = cs.getChangedDataObjects().iterator();
        System.out.print("changed objects");
        while (dataObjects.hasNext()) {
            DataObject current = (DataObject) dataObjects.next();
            String name = current.getType().getName();
            System.out.print(":" + name);
        }
        System.out.println();
    }
}