/*
 * Licensed Materials - Property of IBM
 * pidnumber
 * (C) Copyright IBM Corporation 2005. All Rights Reserved.
 * US Government Users Restricted Rights- Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 */
package com.ibm.j2ca.sample.twineball.outbound;

import java.util.Iterator;
import java.util.List;
import junit.framework.TestCase;
import com.ibm.j2ca.base.AdapterBOUtil;
import com.ibm.j2ca.base.TopLevelVerbs;
import com.ibm.j2ca.base.WBIInteractionSpec;
import com.ibm.j2ca.base.WBIRecord;
import com.ibm.j2ca.extension.logging.LogUtils;
import com.ibm.j2ca.sample.twineball.TwineBallResourceAdapter;
import com.ibm.j2ca.sample.twineball.mocks.MockBootstrapContext;
import com.ibm.j2ca.sample.twineball.mocks.SDOBuilder;
import commonj.sdo.DataObject;

public class InteractionTest extends TestCase {
    /*
     * @see TestCase#setUp()
     */
    private static final String CUSTOMERBG = SDOBuilder.GRAPH_NAME;
    private static final String FIRSTNAME = SDOBuilder.FIRSTNAME;
    private static final String ADDRESS = SDOBuilder.cardNChild;
    private static final String PRIMARYKEY = SDOBuilder.KEY_FIELD;
    private static final String CUSTOMER = SDOBuilder.PRIMARY_DATA_OBJECT;
    TwineBallManagedConnectionFactory managedConnectionFactory = null;
    javax.resource.cci.ConnectionFactory connectionFactory = null;
    javax.resource.cci.Connection connection = null;
    javax.resource.cci.Interaction interaction = null;
    javax.resource.cci.ConnectionSpec connectionSpec = null;
    WBIInteractionSpec interactionSpec = null;
    SDOBuilder builder = null;

    protected void setUp() throws Exception {
        super.setUp();
        builder = new SDOBuilder();
        // Get a ConnectionFactory instance:
        managedConnectionFactory = new TwineBallManagedConnectionFactory();
        TwineBallResourceAdapter ra = new TwineBallResourceAdapter();
        String URL = "Cloudscape";
        String userId = "Administrator";
        String password = "media1a";
        managedConnectionFactory.setURL(URL);
        managedConnectionFactory.setSchema("twineball");
        ra.setLogFilename("LOG.LOG");
        ra.setTraceFilename("TRACE.TRACE");
        LogUtils utils = new LogUtils(ra);
        ra.start(new MockBootstrapContext());
        managedConnectionFactory.setResourceAdapter(ra);
        connectionSpec = new TwineBallConnectionSpec(userId, password);
        connectionFactory = (javax.resource.cci.ConnectionFactory) managedConnectionFactory.createConnectionFactory();
        connection = connectionFactory.getConnection(connectionSpec);
        // Set up Interaction
        interaction = connection.createInteraction();
        interactionSpec = new WBIInteractionSpec();
        interactionSpec.setFunctionName(interactionSpec.APPLY_CHANGES_OP);
    }

    /*
     * @see TestCase#tearDown()
     */
    protected void tearDown() throws Exception {
        super.tearDown();
    }

    public void testDeleteSingleLevel() throws Exception {
        WBIRecord ioRecord = new WBIRecord();
        DataObject graph = builder.createCustomer("0001");
        AdapterBOUtil.markAsAfterImageObject(graph, TopLevelVerbs.DELETE_TLV);
        ioRecord.setDataObject(graph);
        javax.resource.cci.LocalTransaction localTran = connection.getLocalTransaction();
        localTran.begin();
        WBIRecord outRecord = (WBIRecord) interaction.execute(interactionSpec, ioRecord);
        localTran.commit();
    }


    public void testInsertSingleLevel() throws Exception {
        WBIRecord ioRecord = new WBIRecord();
        DataObject graph = builder.createCustomer("0001");
        ioRecord.setDataObject(graph);
        AdapterBOUtil.markAsAfterImageObject(graph, TopLevelVerbs.CREATE_TLV);
        DataObject dataObj = builder.getCustomer(graph);
        dataObj.setString(SDOBuilder.FIRSTNAME, "travis2");
        dataObj.setString(SDOBuilder.LASTNAME, "Nelson");
        javax.resource.cci.LocalTransaction localTran = connection.getLocalTransaction();
        localTran.begin();
        WBIRecord outRecord = (WBIRecord) interaction.execute(interactionSpec, ioRecord);
        localTran.commit();
        DataObject returnedObject = outRecord.getDataObject();
        dataObj = returnedObject.getDataObject(SDOBuilder.PRIMARY_DATA_OBJECT);
        String key = dataObj.getString(SDOBuilder.KEY_FIELD);
        assertTrue(null != key);
        WBIRecord retrieved = retrieve(key);
        graph = retrieved.getDataObject();
        DataObject retrievedDataObject = graph.getDataObject(SDOBuilder.PRIMARY_DATA_OBJECT);
        assertTrue(retrievedDataObject.getString(SDOBuilder.FIRSTNAME).equals("travis2"));
    }

    public void testInsertMultiLevel() throws Exception {
        WBIRecord ioRecord = new WBIRecord();
        DataObject dataObject = builder.createCustomer("00001");
        AdapterBOUtil.markAsAfterImageObject(dataObject, TopLevelVerbs.CREATE_TLV);
        builder.addNAddresses(dataObject, 3);
        ioRecord.setDataObject(dataObject);
        javax.resource.cci.LocalTransaction localTran = connection.getLocalTransaction();
        localTran.begin();
        WBIRecord outRecord = (WBIRecord) interaction.execute(interactionSpec, ioRecord);
        DataObject returnedObject = outRecord.getDataObject();
        DataObject dataObj = returnedObject.getDataObject(SDOBuilder.PRIMARY_DATA_OBJECT);
        String key = dataObj.getString(SDOBuilder.KEY_FIELD);
        localTran.commit();
        WBIRecord retrieved = retrieve(key);
        DataObject wrapper = retrieved.getDataObject();
        DataObject retrievedDataObject = wrapper.getDataObject(SDOBuilder.PRIMARY_DATA_OBJECT);
        assertTrue(retrievedDataObject.getString(SDOBuilder.FIRSTNAME).equals("Travis"));
    }

    public void testAfterImageUpdate() throws Exception {
        WBIRecord ioRecord = new WBIRecord();
        DataObject dataObject = builder.createCustomer("00001");
        AdapterBOUtil.markAsAfterImageObject(dataObject, TopLevelVerbs.CREATE_TLV);
        builder.addNAddresses(dataObject, 1);
        ioRecord.setDataObject(dataObject);
        javax.resource.cci.LocalTransaction localTran = connection.getLocalTransaction();
        localTran.begin();
        WBIRecord outRecord = (WBIRecord) interaction.execute(interactionSpec, ioRecord);
        DataObject returnedObject = outRecord.getDataObject();
        DataObject customerObj = returnedObject.getDataObject(CUSTOMER);
        String key = customerObj.getString(PRIMARYKEY);
        localTran.commit();
        DataObject updateBO = builder.createCustomer("00001");
        builder.addNAddresses(updateBO, 3);
        DataObject customerObj2 = updateBO.getDataObject(CUSTOMER);
        AdapterBOUtil.markAsAfterImageObject(updateBO, TopLevelVerbs.UPDATE_TLV);
        customerObj2.setString(PRIMARYKEY, key);
        ioRecord.setDataObject(updateBO);
        localTran.begin();
        outRecord = (WBIRecord) interaction.execute(interactionSpec, ioRecord);
        returnedObject = outRecord.getDataObject();
        localTran.commit();
        WBIRecord retrievedRecord = retrieve(key);
        returnedObject = retrievedRecord.getDataObject();
        DataObject retrievedDataObject = returnedObject.getDataObject(CUSTOMER);
        List objectList = retrievedDataObject.getList(ADDRESS);
        assertEquals(3, objectList.size());
    }

    public void testAfterImageUpdateMissingChild() throws Exception {
        WBIRecord ioRecord = new WBIRecord();
        DataObject dataObject = builder.createCustomer("00001");
        AdapterBOUtil.markAsAfterImageObject(dataObject, TopLevelVerbs.CREATE_TLV);
        builder.addNAddresses(dataObject, 2);
        ioRecord.setDataObject(dataObject);
        javax.resource.cci.LocalTransaction localTran = connection.getLocalTransaction();
        localTran.begin();
        WBIRecord outRecord = (WBIRecord) interaction.execute(interactionSpec, ioRecord);
        DataObject returnedObject = outRecord.getDataObject();
        DataObject customerObj = returnedObject.getDataObject(CUSTOMER);
        String key = customerObj.getString(PRIMARYKEY);
        localTran.commit();
        DataObject updateBO = builder.createCustomer("00001");
        builder.addNAddresses(updateBO, 1);
        DataObject customerObj2 = updateBO.getDataObject(CUSTOMER);
        AdapterBOUtil.markAsAfterImageObject(updateBO, TopLevelVerbs.UPDATE_TLV);
        customerObj2.setString(PRIMARYKEY, key);
        ioRecord.setDataObject(updateBO);
        localTran.begin();
        outRecord = (WBIRecord) interaction.execute(interactionSpec, ioRecord);
        returnedObject = outRecord.getDataObject();
        localTran.commit();
        WBIRecord retrievedRecord = retrieve(key);
        returnedObject = retrievedRecord.getDataObject();
        DataObject retrievedDataObject = returnedObject.getDataObject(CUSTOMER);
        List objectList = retrievedDataObject.getList(ADDRESS);
        assertEquals(1, objectList.size());
    }

    public static void logTrace(String inString) {
        System.out.println(inString);
    }

    public WBIRecord retrieve(String key) throws Exception {
        interactionSpec.setFunctionName(interactionSpec.RETRIEVE_OP);
        WBIRecord ioRecord = new WBIRecord();
        DataObject dataObject = builder.createCustomer("00001");
        DataObject customer = builder.getCustomer(dataObject);
        customer.setString(PRIMARYKEY, key);
        ioRecord.setDataObject(dataObject);
        WBIRecord outRecord = (WBIRecord) interaction.execute(interactionSpec, ioRecord);
        return outRecord;
    }


}